@@ -13,12 +13,12 @@ __docformat__ = 'restructuredtext'
 # and by Travis Oliphant  2005-8-22 for numpy
 
 import sys
-import numeric      as _nc
 import numerictypes as _nt
 from umath import maximum, minimum, absolute, not_equal, isnan, isinf
 from multiarray import format_longfloat
 from fromnumeric import ravel
 
+
 def product(x, y): return x*y
 
 _summaryEdgeItems = 3     # repr N leading and trailing items of each dimension
@@ -27,9 +27,11 @@ _summaryThreshold = 1000 # total items > triggers array summarization
 _float_output_precision = 8
 _float_output_suppress_small = False
 _line_width = 75
-_nan_str = 'NaN'
-_inf_str = 'Inf'
+_nan_str = 'nan'
+_inf_str = 'inf'
 
+if sys.version_info[0] >= 3:
+    from functools import reduce
 
 def set_printoptions(precision=None, threshold=None, edgeitems=None,
                      linewidth=None, suppress=None,
@@ -56,11 +58,15 @@ def set_printoptions(precision=None, threshold=None, edgeitems=None,
     suppress : bool, optional
         Whether or not suppress printing of small floating point values
         using scientific notation (default False).
-    nanstr : string, optional
+    nanstr : str, optional
         String representation of floating point not-a-number (default nan).
-    infstr : string, optional
+    infstr : str, optional
         String representation of floating point infinity (default inf).
 
+    See Also
+    --------
+    get_printoptions, set_string_function
+
     Examples
     --------
     Floating point precision can be set:
@@ -79,15 +85,17 @@ def set_printoptions(precision=None, threshold=None, edgeitems=None,
 
     >>> eps = np.finfo(float).eps
     >>> x = np.arange(4.)
-
     >>> x**2 - (x + eps)**2
     array([ -4.9304e-32,  -4.4409e-16,   0.0000e+00,   0.0000e+00])
-
     >>> np.set_printoptions(suppress=True)
-
     >>> x**2 - (x + eps)**2
     array([-0., -0.,  0.,  0.])
 
+    To put back the default options, you can use:
+
+    >>> np.set_printoptions(edgeitems=3,infstr='Inf',
+    ... linewidth=75, nanstr='NaN', precision=8,
+    ... suppress=False, threshold=1000)
     """
 
     global _summaryThreshold, _summaryEdgeItems, _float_output_precision, \
@@ -121,12 +129,14 @@ def get_printoptions():
           - edgeitems : int
           - linewidth : int
           - suppress : bool
-          - nanstr : string
-          - infstr : string
+          - nanstr : str
+          - infstr : str
+
+        For a full description of these options, see `set_printoptions`.
 
     See Also
     --------
-    set_printoptions : parameter descriptions
+    set_printoptions, set_string_function
 
     """
     d = dict(precision=_float_output_precision,
@@ -139,6 +149,7 @@ def get_printoptions():
     return d
 
 def _leading_trailing(a):
+    import numeric as _nc
     if a.ndim == 1:
         if len(a) > 2*_summaryEdgeItems:
             b = _nc.concatenate((a[:_summaryEdgeItems],
@@ -188,18 +199,21 @@ def _array2string(a, max_line_width, precision, suppress_small, separator=' ',
             # make sure True and False line up.
             format_function = _boolFormatter
         elif issubclass(dtypeobj, _nt.integer):
-            max_str_len = max(len(str(maximum.reduce(data))),
-                              len(str(minimum.reduce(data))))
-            format = '%' + str(max_str_len) + 'd'
-            format_function = lambda x: _formatInteger(x, format)
+            if issubclass(dtypeobj, _nt.timeinteger):
+                format_function = str
+            else:
+                max_str_len = max(len(str(maximum.reduce(data))),
+                                  len(str(minimum.reduce(data))))
+                format = '%' + str(max_str_len) + 'd'
+                format_function = lambda x: _formatInteger(x, format)
         elif issubclass(dtypeobj, _nt.floating):
             if issubclass(dtypeobj, _nt.longfloat):
-                format_function = _longfloatFormatter(precision)
+                format_function = LongFloatFormat(precision)
             else:
                 format_function = FloatFormat(data, precision, suppress_small)
         elif issubclass(dtypeobj, _nt.complexfloating):
             if issubclass(dtypeobj, _nt.clongfloat):
-                format_function = _clongfloatFormatter(precision)
+                format_function = LongComplexFormat(precision)
             else:
                 format_function = ComplexFormat(data, precision, suppress_small)
         elif issubclass(dtypeobj, _nt.unicode_) or \
@@ -218,6 +232,7 @@ def _array2string(a, max_line_width, precision, suppress_small, separator=' ',
     return lst
 
 def _convert_arrays(obj):
+    import numeric as _nc
     newtup = []
     for k in obj:
         if isinstance(k, _nc.ndarray):
@@ -247,9 +262,9 @@ def array2string(a, max_line_width = None, precision = None,
     suppress_small : bool, optional
         Represent very small numbers as zero. A number is "very small" if it
         is smaller than the current printing precision.
-    separator : string, optional
+    separator : str, optional
         Inserted between elements.
-    prefix : string, optional
+    prefix : str, optional
         An array is typically printed as::
 
           'prefix(' + array2string(a) + ')'
@@ -380,6 +395,7 @@ class FloatFormat(object):
         self.fillFormat(data)
 
     def fillFormat(self, data):
+        import numeric as _nc
         errstate = _nc.seterr(all='ignore')
         try:
             special = isnan(data) | isinf(data)
@@ -397,6 +413,7 @@ class FloatFormat(object):
                     self.exp_format = True
         finally:
             _nc.seterr(**errstate)
+
         if self.exp_format:
             self.large_exponent = 0 < min_val < 1e-99 or max_val >= 1e100
             self.max_str_len = 8 + self.precision
@@ -425,17 +442,30 @@ class FloatFormat(object):
             else:
                 format = '%#'
             format = format + '%d.%df' % (self.max_str_len, precision)
+
         self.special_fmt = '%%%ds' % (self.max_str_len,)
         self.format = format
 
     def __call__(self, x, strip_zeros=True):
-        if isnan(x):
-            return self.special_fmt % (_nan_str,)
-        elif isinf(x):
-            if x > 0:
-                return self.special_fmt % (_inf_str,)
-            else:
-                return self.special_fmt % ('-' + _inf_str,)
+        import numeric as _nc
+        err = _nc.seterr(invalid='ignore')
+        try:
+            if isnan(x):
+                if self.sign:
+                    return self.special_fmt % ('+' + _nan_str,)
+                else:
+                    return self.special_fmt % (_nan_str,)
+            elif isinf(x):
+                if x > 0:
+                    if self.sign:
+                        return self.special_fmt % ('+' + _inf_str,)
+                    else:
+                        return self.special_fmt % (_inf_str,)
+                else:
+                    return self.special_fmt % ('-' + _inf_str,)
+        finally:
+            _nc.seterr(**err)
+
         s = self.format % x
         if self.large_exponent:
             # 3-digit exponent
@@ -466,26 +496,46 @@ def _formatInteger(x, format):
     else:
         return "%s" % x
 
-def _longfloatFormatter(precision):
+class LongFloatFormat(object):
     # XXX Have to add something to determine the width to use a la FloatFormat
     # Right now, things won't line up properly
-    def formatter(x):
+    def __init__(self, precision, sign=False):
+        self.precision = precision
+        self.sign = sign
+
+    def __call__(self, x):
         if isnan(x):
-            return _nan_str
+            if self.sign:
+                return '+' + _nan_str
+            else:
+                return ' ' + _nan_str
         elif isinf(x):
             if x > 0:
-                return _inf_str
+                if self.sign:
+                    return '+' + _inf_str
+                else:
+                    return ' ' + _inf_str
             else:
                 return '-' + _inf_str
-        return format_longfloat(x, precision)
-    return formatter
+        elif x >= 0:
+            if self.sign:
+                return '+' + format_longfloat(x, self.precision)
+            else:
+                return ' ' + format_longfloat(x, self.precision)
+        else:
+            return format_longfloat(x, self.precision)
+
+
+class LongComplexFormat(object):
+    def __init__(self, precision):
+        self.real_format = LongFloatFormat(precision)
+        self.imag_format = LongFloatFormat(precision, sign=True)
+
+    def __call__(self, x):
+        r = self.real_format(x.real)
+        i = self.imag_format(x.imag)
+        return r + i + 'j'
 
-def _clongfloatFormatter(precision):
-    def formatter(x):
-        r = format_longfloat(x.real, precision)
-        i = format_longfloat(x.imag, precision)
-        return '%s+%sj' % (r, i)
-    return formatter
 
 class ComplexFormat(object):
     def __init__(self, x, precision, suppress_small):
