@@ -10,18 +10,22 @@
     /* Use endian.h if available */
     #include <endian.h>
 
-    #define NPY_BYTE_ORDER __BYTE_ORDER
-    #if (__BYTE_ORDER == __LITTLE_ENDIAN)
-        #define NPY_LITTLE_ENDIAN
-    #elif (__BYTE_ORDER == __BIG_ENDIAN)
-        #define NPY_BIG_ENDIAN
-    #else
-        #error Unknown machine endianness detected.
-    #endif
+#   if defined(BYTE_ORDER)
+#      define NPY_BYTE_ORDER BYTE_ORDER
+#      define NPY_LITTLE_ENDIAN LITTLE_ENDIAN
+#      define NPY_BIG_ENDIAN BIG_ENDIAN
+#   else
+#      define NPY_BYTE_ORDER __BYTE_ORDER
+#      define NPY_LITTLE_ENDIAN __LITTLE_ENDIAN
+#      define NPY_BIG_ENDIAN __BIG_ENDIAN
+#   endif
 #else
     /* Set endianness info using target CPU */
     #include "npy_cpu.h"
 
+    #define NPY_LITTLE_ENDIAN 1234
+    #define NPY_BIG_ENDIAN 4321
+
     #if defined(NPY_CPU_X86)            \
             || defined(NPY_CPU_AMD64)   \
             || defined(NPY_CPU_IA64)    \
@@ -29,7 +33,7 @@
             || defined(NPY_CPU_ARMEL)   \
             || defined(NPY_CPU_SH_LE)   \
             || defined(NPY_CPU_MIPSEL)
-        #define NPY_BYTE_ORDER 1234
+        #define NPY_BYTE_ORDER NPY_LITTLE_ENDIAN
     #elif defined(NPY_CPU_PPC)          \
             || defined(NPY_CPU_SPARC)   \
             || defined(NPY_CPU_S390)    \
@@ -38,7 +42,7 @@
             || defined(NPY_CPU_ARMEB)   \
             || defined(NPY_CPU_SH_BE)   \
             || defined(NPY_CPU_MIPSEB)
-        #define NPY_BYTE_ORDER 4321
+        #define NPY_BYTE_ORDER NPY_BIG_ENDIAN
     #else
         #error Unknown CPU: can not set endianness
     #endif
