@@ -1,6 +1,7 @@
-"""numerictypes: Define the numeric type objects
+"""
+numerictypes: Define the numeric type objects
 
-This module is designed so 'from numerictypes import *' is safe.
+This module is designed so "from numerictypes import \\*" is safe.
 Exported symbols include:
 
   Dictionary with all registered number types (including aliases):
@@ -15,6 +16,7 @@ Exported symbols include:
     uint8 uint16 uint32 uint64 uint128
     float16 float32 float64 float96 float128 float256
     complex32 complex64 complex128 complex192 complex256 complex512
+    datetime64 timedelta64
 
     c-based names
 
@@ -31,10 +33,16 @@ Exported symbols include:
     int_, uint,
     longlong, ulonglong,
 
+
     single, csingle,
     float_, complex_,
     longfloat, clongfloat,
 
+
+    datetime_, timedelta_,  (these inherit from timeinteger which inherits
+    from signedinteger)
+
+
    As part of the type-hierarchy:    xx -- is bit-width
 
    generic
@@ -57,34 +65,48 @@ Exported symbols include:
      |     ulonglong
      +-> inexact
      |   +-> floating           (floatxx)       (kind=f)
+     |   |     half
      |   |     single
      |   |     float_  (double)
      |   |     longfloat
-     |   \-> complexfloating    (complexxx)     (kind=c)
+     |   \\-> complexfloating    (complexxx)     (kind=c)
      |         csingle  (singlecomplex)
      |         complex_ (cfloat, cdouble)
      |         clongfloat (longcomplex)
      +-> flexible
      |     character
-     |     str_     (string_)                   (kind=S)
-     |     unicode_                             (kind=U)
      |     void                                 (kind=V)
      |
-     \-> object_ (not used much)                (kind=O)
+     |     str_     (string_, bytes_)           (kind=S)    [Python 2]
+     |     unicode_                             (kind=U)    [Python 2]
+     |
+     |     bytes_   (string_)                   (kind=S)    [Python 3]
+     |     str_     (unicode_)                  (kind=U)    [Python 3]
+     |
+     \\-> object_ (not used much)                (kind=O)
+
 """
 
 # we add more at the bottom
 __all__ = ['sctypeDict', 'sctypeNA', 'typeDict', 'typeNA', 'sctypes',
            'ScalarType', 'obj2sctype', 'cast', 'nbytes', 'sctype2char',
-           'maximum_sctype', 'issctype', 'typecodes', 'find_common_type']
+           'maximum_sctype', 'issctype', 'typecodes', 'find_common_type',
+           'issubdtype']
 
 from numpy.core.multiarray import typeinfo, ndarray, array, empty, dtype
 import types as _types
+import sys
 
 # we don't export these for import *, but we do want them accessible
 # as numerictypes.bool, etc.
 from __builtin__ import bool, int, long, float, complex, object, unicode, str
+from numpy.compat import bytes
 
+if sys.version_info[0] >= 3:
+    # Py3K
+    class long(int):
+        # Placeholder class -- this will not escape outside numerictypes.py
+        pass
 
 # String-handling utilities to avoid locale-dependence.
 
@@ -228,12 +250,6 @@ def bitname(obj):
     if name == 'bool_':
         char = 'b'
         base = 'bool'
-    elif name=='string_':
-        char = 'S'
-        base = 'string'
-    elif name=='unicode_':
-        char = 'U'
-        base = 'unicode'
     elif name=='void':
         char = 'V'
         base = 'void'
@@ -242,7 +258,22 @@ def bitname(obj):
         base = 'object'
         bits = 0
 
-    bytes = bits / 8
+    if sys.version_info[0] >= 3:
+        if name=='bytes_':
+            char = 'S'
+            base = 'bytes'
+        elif name=='str_':
+            char = 'U'
+            base = 'str'
+    else:
+        if name=='string_':
+            char = 'S'
+            base = 'string'
+        elif name=='unicode_':
+            char = 'U'
+            base = 'unicode'
+
+    bytes = bits // 8
 
     if char != '' and bytes != 0:
         char = "%s%d" % (char, bytes)
@@ -282,7 +313,7 @@ def _add_aliases():
                 allTypes[myname] = typeobj
                 sctypeDict[myname] = typeobj
                 if base == 'complex':
-                    na_name = '%s%d' % (english_capitalize(base), bit/2)
+                    na_name = '%s%d' % (english_capitalize(base), bit//2)
                 elif base == 'bool':
                     na_name = english_capitalize(base)
                     sctypeDict[na_name] = typeobj
@@ -312,8 +343,8 @@ def _add_integer_aliases():
     for ctype in _ctypes:
         val = typeinfo[ctype]
         bits = val[2]
-        charname = 'i%d' % (bits/8,)
-        ucharname = 'u%d' % (bits/8,)
+        charname = 'i%d' % (bits//8,)
+        ucharname = 'u%d' % (bits//8,)
         intname = 'int%d' % bits
         UIntname = 'UInt%d' % bits
         Intname = 'Int%d' % bits
@@ -366,15 +397,30 @@ def _set_up_aliases():
                   ('longcomplex', 'clongdouble'),
                   ('bool_', 'bool'),
                   ('unicode_', 'unicode'),
-                  ('str_', 'string'),
-                  ('string_', 'string'),
-                  ('object_', 'object')]
+                  ('object_', 'object'),
+                  ('timedelta_', 'timedelta'),
+                  ('datetime_', 'datetime')]
+    if sys.version_info[0] >= 3:
+        type_pairs.extend([('bytes_', 'string'),
+                           ('str_', 'unicode'),
+                           ('string_', 'string')])
+    else:
+        type_pairs.extend([('str_', 'string'),
+                           ('string_', 'string'),
+                           ('bytes_', 'string')])
     for alias, t in type_pairs:
         allTypes[alias] = allTypes[t]
         sctypeDict[alias] = sctypeDict[t]
     # Remove aliases overriding python types and modules
-    for t in ['ulong', 'object', 'unicode', 'int', 'long', 'float',
-              'complex', 'bool', 'string']:
+    to_remove = ['ulong', 'object', 'unicode', 'int', 'long', 'float',
+                 'complex', 'bool', 'string', 'datetime', 'timedelta']
+    if sys.version_info[0] >= 3:
+        # Py3K
+        to_remove.append('bytes')
+        to_remove.append('str')
+        to_remove.remove('unicode')
+        to_remove.remove('long')
+    for t in to_remove:
         try:
             del allTypes[t]
             del sctypeDict[t]
@@ -439,7 +485,43 @@ genericTypeRank = ['bool', 'int8', 'uint8', 'int16', 'uint16',
                    'complex192', 'complex256', 'complex512', 'object']
 
 def maximum_sctype(t):
-    """returns the sctype of highest precision of the same general kind as 't'"""
+    """
+    Return the scalar type of highest precision of the same kind as the input.
+
+    Parameters
+    ----------
+    t : dtype or dtype specifier
+        The input data type. This can be a `dtype` object or an object that
+        is convertible to a `dtype`.
+
+    Returns
+    -------
+    out : dtype
+        The highest precision data type of the same kind (`dtype.kind`) as `t`.
+
+    See Also
+    --------
+    obj2sctype, mintypecode, sctype2char
+    dtype
+
+    Examples
+    --------
+    >>> np.maximum_sctype(np.int)
+    <type 'numpy.int64'>
+    >>> np.maximum_sctype(np.uint8)
+    <type 'numpy.uint64'>
+    >>> np.maximum_sctype(np.complex)
+    <type 'numpy.complex192'>
+
+    >>> np.maximum_sctype(str)
+    <type 'numpy.string_'>
+
+    >>> np.maximum_sctype('i2')
+    <type 'numpy.int64'>
+    >>> np.maximum_sctype('f4')
+    <type 'numpy.float96'>
+
+    """
     g = obj2sctype(t)
     if g is None:
         return t
@@ -451,23 +533,68 @@ def maximum_sctype(t):
     else:
         return sctypes[base][-1]
 
+try:
+    buffer_type = _types.BufferType
+except AttributeError:
+    # Py3K
+    buffer_type = memoryview
+
 _python_types = {int : 'int_',
                  float: 'float_',
                  complex: 'complex_',
                  bool: 'bool_',
-                 str: 'string_',
+                 bytes: 'bytes_',
                  unicode: 'unicode_',
-                 _types.BufferType: 'void',
+                 buffer_type: 'void',
                 }
-def _python_type(t):
-    """returns the type corresponding to a certain Python type"""
-    if not isinstance(t, _types.TypeType):
-        t = type(t)
-    return allTypes[_python_types.get(t, 'object_')]
+
+if sys.version_info[0] >= 3:
+    def _python_type(t):
+        """returns the type corresponding to a certain Python type"""
+        if not isinstance(t, type):
+            t = type(t)
+        return allTypes[_python_types.get(t, 'object_')]
+else:
+    def _python_type(t):
+        """returns the type corresponding to a certain Python type"""
+        if not isinstance(t, _types.TypeType):
+            t = type(t)
+        return allTypes[_python_types.get(t, 'object_')]
 
 def issctype(rep):
-    """Determines whether the given object represents
-    a numeric array type."""
+    """
+    Determines whether the given object represents a scalar data-type.
+
+    Parameters
+    ----------
+    rep : any
+        If `rep` is an instance of a scalar dtype, True is returned. If not,
+        False is returned.
+
+    Returns
+    -------
+    out : bool
+        Boolean result of check whether `rep` is a scalar dtype.
+
+    See Also
+    --------
+    issubsctype, issubdtype, obj2sctype, sctype2char
+
+    Examples
+    --------
+    >>> np.issctype(np.int32)
+    True
+    >>> np.issctype(list)
+    False
+    >>> np.issctype(1.1)
+    False
+
+    Strings are also a scalar type:
+
+    >>> np.issctype(np.dtype('str'))
+    True
+
+    """
     if not isinstance(rep, (type, dtype)):
         return False
     try:
@@ -479,6 +606,44 @@ def issctype(rep):
         return False
 
 def obj2sctype(rep, default=None):
+    """
+    Return the scalar dtype or NumPy equivalent of Python type of an object.
+
+    Parameters
+    ----------
+    rep : any
+        The object of which the type is returned.
+    default : any, optional
+        If given, this is returned for objects whose types can not be
+        determined. If not given, None is returned for those objects.
+
+    Returns
+    -------
+    dtype : dtype or Python type
+        The data type of `rep`.
+
+    See Also
+    --------
+    sctype2char, issctype, issubsctype, issubdtype, maximum_sctype
+
+    Examples
+    --------
+    >>> np.obj2sctype(np.int32)
+    <type 'numpy.int32'>
+    >>> np.obj2sctype(np.array([1., 2.]))
+    <type 'numpy.float64'>
+    >>> np.obj2sctype(np.array([1.j]))
+    <type 'numpy.complex128'>
+
+    >>> np.obj2sctype(dict)
+    <type 'numpy.object_'>
+    >>> np.obj2sctype('string')
+    <type 'numpy.string_'>
+
+    >>> np.obj2sctype(1, default=list)
+    <type 'list'>
+
+    """
     try:
         if issubclass(rep, generic):
             return rep
@@ -498,12 +663,71 @@ def obj2sctype(rep, default=None):
 
 
 def issubclass_(arg1, arg2):
+    """
+    Determine if a class is a subclass of a second class.
+
+    `issubclass_` is equivalent to the Python built-in ``issubclass``,
+    except that it returns False instead of raising a TypeError is one
+    of the arguments is not a class.
+
+    Parameters
+    ----------
+    arg1 : class
+        Input class. True is returned if `arg1` is a subclass of `arg2`.
+    arg2 : class or tuple of classes.
+        Input class. If a tuple of classes, True is returned if `arg1` is a
+        subclass of any of the tuple elements.
+
+    Returns
+    -------
+    out : bool
+        Whether `arg1` is a subclass of `arg2` or not.
+
+    See Also
+    --------
+    issubsctype, issubdtype, issctype
+
+    Examples
+    --------
+    >>> np.issubclass_(np.int32, np.int)
+    True
+    >>> np.issubclass_(np.int32, np.float)
+    False
+
+    """
     try:
         return issubclass(arg1, arg2)
     except TypeError:
         return False
 
 def issubsctype(arg1, arg2):
+    """
+    Determine if the first argument is a subclass of the second argument.
+
+    Parameters
+    ----------
+    arg1, arg2 : dtype or dtype specifier
+        Data-types.
+
+    Returns
+    -------
+    out : bool
+        The result.
+
+    See Also
+    --------
+    issctype, issubdtype,obj2sctype
+
+    Examples
+    --------
+    >>> np.issubsctype('S8', str)
+    True
+    >>> np.issubsctype(np.array([1]), np.int)
+    True
+    >>> np.issubsctype(np.array([1]), np.float)
+    False
+
+    """
     return issubclass(obj2sctype(arg1), obj2sctype(arg2))
 
 def issubdtype(arg1, arg2):
@@ -512,14 +736,16 @@ def issubdtype(arg1, arg2):
 
     Parameters
     ----------
-    arg1 : dtype_like
-        dtype or string representing a typecode.
-    arg2 : dtype_like
+    arg1, arg2 : dtype_like
         dtype or string representing a typecode.
 
+    Returns
+    -------
+    out : bool
 
     See Also
     --------
+    issubsctype, issubclass_
     numpy.core.numerictypes : Overview of numpy type hierarchy.
 
     Examples
@@ -542,6 +768,13 @@ def issubdtype(arg1, arg2):
 
 # This dictionary allows look up based on any alias for an array data-type
 class _typedict(dict):
+    """
+    Base object for a dictionary for look-up with any alias for an array dtype.
+
+    Instances of `_typedict` can not be used as dictionaries directly,
+    first they have to be populated.
+
+    """
     def __getitem__(self, obj):
         return dict.__getitem__(self, obj2sctype(obj))
 
@@ -554,7 +787,7 @@ def _construct_lookups():
         if not isinstance(val, tuple):
             continue
         obj = val[-1]
-        nbytes[obj] = val[2] / 8
+        nbytes[obj] = val[2] // 8
         _alignment[obj] = val[3]
         if (len(val) > 5):
             _maxvals[obj] = val[4]
@@ -566,6 +799,47 @@ def _construct_lookups():
 _construct_lookups()
 
 def sctype2char(sctype):
+    """
+    Return the string representation of a scalar dtype.
+
+    Parameters
+    ----------
+    sctype : scalar dtype or object
+        If a scalar dtype, the corresponding string character is
+        returned. If an object, `sctype2char` tries to infer its scalar type
+        and then return the corresponding string character.
+
+    Returns
+    -------
+    typechar : str
+        The string character corresponding to the scalar type.
+
+    Raises
+    ------
+    ValueError
+        If `sctype` is an object for which the type can not be inferred.
+
+    See Also
+    --------
+    obj2sctype, issctype, issubsctype, mintypecode
+
+    Examples
+    --------
+    >>> for sctype in [np.int32, np.float, np.complex, np.string_, np.ndarray]:
+    ...     print np.sctype2char(sctype)
+    l
+    d
+    D
+    S
+    O
+
+    >>> x = np.array([1., 2-1.j])
+    >>> np.sctype2char(x)
+    'D'
+    >>> np.sctype2char(list)
+    'O'
+
+    """
     sctype = obj2sctype(sctype)
     if sctype is None:
         raise ValueError, "unrecognized type"
@@ -576,9 +850,14 @@ def sctype2char(sctype):
 
 
 cast = _typedict()
-ScalarType = [_types.IntType, _types.FloatType,
-              _types.ComplexType, _types.LongType, _types.BooleanType,
-              _types.StringType, _types.UnicodeType, _types.BufferType]
+try:
+    ScalarType = [_types.IntType, _types.FloatType, _types.ComplexType,
+                  _types.LongType, _types.BooleanType,
+                   _types.StringType, _types.UnicodeType, _types.BufferType]
+except AttributeError:
+    # Py3K
+    ScalarType = [int, float, complex, long, bool, bytes, str, memoryview]
+
 ScalarType.extend(_sctype2char_dict.keys())
 ScalarType = tuple(ScalarType)
 for key in _sctype2char_dict.keys():
@@ -599,8 +878,13 @@ for key, val in _typestr.items():
 
 # Add additional strings to the sctypeDict
 
-_toadd = ['int', 'float', 'complex', 'bool', 'object', 'string', ('str', allTypes['string_']),
-          'unicode', 'object', ('a', allTypes['string_'])]
+if sys.version_info[0] >= 3:
+    _toadd = ['int', 'float', 'complex', 'bool', 'object',
+              'str', 'bytes', 'object', ('a', allTypes['bytes_'])]
+else:
+    _toadd = ['int', 'float', 'complex', 'bool', 'object', 'string',
+              ('str', allTypes['string_']),
+              'unicode', 'object', ('a', allTypes['string_'])]
 
 for name in _toadd:
     if isinstance(name, tuple):
@@ -620,11 +904,12 @@ del key
 typecodes = {'Character':'c',
              'Integer':'bhilqp',
              'UnsignedInteger':'BHILQP',
-             'Float':'fdg',
+             'Float':'efdg',
              'Complex':'FDG',
              'AllInteger':'bBhHiIlLqQpP',
-             'AllFloat':'fdgFDG',
-             'All':'?bhilqpBHILQPfdgFDGSUVO'}
+             'AllFloat':'efdgFDG',
+             'Datetime': 'Mm',
+             'All':'?bhilqpBHILQPefdgFDGSUVOMm'}
 
 # backwards compatibility --- deprecated name
 typeDict = sctypeDict
@@ -635,13 +920,15 @@ typeNA = sctypeNA
 # i -> signed integer
 # f -> floating point
 # c -> complex
+# M -> datetime
+# m -> timedelta
 # S -> string
 # U -> Unicode string
 # V -> record
 # O -> Python object
-_kind_list = ['b', 'u', 'i', 'f', 'c', 'S', 'U', 'V', 'O']
+_kind_list = ['b', 'u', 'i', 'f', 'c', 'S', 'U', 'V', 'O', 'M', 'm']
 
-__test_types = typecodes['AllInteger'][:-2]+typecodes['AllFloat']+'O'
+__test_types = '?'+typecodes['AllInteger'][:-2]+typecodes['AllFloat']+'O'
 __len_test_types = len(__test_types)
 
 # Keep incrementing until a common type both can be coerced to
@@ -653,52 +940,86 @@ def _find_common_coerce(a, b):
         thisind = __test_types.index(a.char)
     except ValueError:
         return None
+    return _can_coerce_all([a,b], start=thisind)
+
+# Find a data-type that all data-types in a list can be coerced to
+def _can_coerce_all(dtypelist, start=0):
+    N = len(dtypelist)
+    if N == 0:
+        return None
+    if N == 1:
+        return dtypelist[0]
+    thisind = start
     while thisind < __len_test_types:
         newdtype = dtype(__test_types[thisind])
-        if newdtype >= b and newdtype >= a:
+        numcoerce = len([x for x in dtypelist if newdtype >= x])
+        if numcoerce == N:
             return newdtype
         thisind += 1
     return None
 
-
 def find_common_type(array_types, scalar_types):
     """
-    Determine common type following standard coercion rules
+    Determine common type following standard coercion rules.
 
     Parameters
     ----------
     array_types : sequence
-        A list of dtype convertible objects representing arrays
+        A list of dtypes or dtype convertible objects representing arrays.
     scalar_types : sequence
-        A list of dtype convertible objects representing scalars
+        A list of dtypes or dtype convertible objects representing scalars.
 
     Returns
     -------
     datatype : dtype
-        The common data-type which is the maximum of the array_types
-        ignoring the scalar_types unless the maximum of the scalar_types
-        is of a different kind.
-
-        If the kinds is not understood, then None is returned.
+        The common data type, which is the maximum of `array_types` ignoring
+        `scalar_types`, unless the maximum of `scalar_types` is of a
+        different kind (`dtype.kind`). If the kind is not understood, then
+        None is returned.
 
     See Also
     --------
-    dtype
+    dtype, common_type, can_cast, mintypecode
+
+    Examples
+    --------
+    >>> np.find_common_type([], [np.int64, np.float32, np.complex])
+    dtype('complex128')
+    >>> np.find_common_type([np.int64, np.float32], [])
+    dtype('float64')
+
+    The standard casting rules ensure that a scalar cannot up-cast an
+    array unless the scalar is of a fundamentally different kind of data
+    (i.e. under a different hierarchy in the data type hierarchy) then
+    the array:
+
+    >>> np.find_common_type([np.float32], [np.int64, np.float64])
+    dtype('float32')
+
+    Complex is of a different type, so it up-casts the float in the
+    `array_types` argument:
+
+    >>> np.find_common_type([np.float32], [np.complex])
+    dtype('complex128')
+
+    Type specifier strings are convertible to dtypes and can therefore
+    be used instead of dtypes:
+
+    >>> np.find_common_type(['f4', 'f4', 'i4'], ['c8'])
+    dtype('complex128')
 
     """
     array_types = [dtype(x) for x in array_types]
     scalar_types = [dtype(x) for x in scalar_types]
 
-    if len(scalar_types) == 0:
-        if len(array_types) == 0:
-            return None
-        else:
-            return max(array_types)
-    if len(array_types) == 0:
-        return max(scalar_types)
+    maxa = _can_coerce_all(array_types)
+    maxsc = _can_coerce_all(scalar_types)
 
-    maxa = max(array_types)
-    maxsc = max(scalar_types)
+    if maxa is None:
+        return maxsc
+
+    if maxsc is None:
+        return maxa
 
     try:
         index_a = _kind_list.index(maxa.kind)
