@@ -3,6 +3,7 @@
 
 import os
 from glob import glob
+import shutil
 from distutils.command.build_clib import build_clib as old_build_clib
 from distutils.errors import DistutilsSetupError, DistutilsError, \
      DistutilsFileError
@@ -27,11 +28,15 @@ class build_clib(old_build_clib):
     user_options = old_build_clib.user_options + [
         ('fcompiler=', None,
          "specify the Fortran compiler type"),
+        ('inplace', 'i', 'Build in-place'),
         ]
 
+    boolean_options = old_build_clib.boolean_options + ['inplace']
+
     def initialize_options(self):
         old_build_clib.initialize_options(self)
         self.fcompiler = None
+        self.inplace = 0
         return
 
     def have_f_sources(self):
@@ -52,9 +57,11 @@ class build_clib(old_build_clib):
 
         # Make sure that library sources are complete.
         languages = []
+
+        # Make sure that extension sources are complete.
+        self.run_command('build_src')
+
         for (lib_name, build_info) in self.libraries:
-            if not all_strings(build_info.get('sources',[])):
-                self.run_command('build_src')
             l = build_info.get('language',None)
             if l and l not in languages: languages.append(l)
 
@@ -92,6 +99,14 @@ class build_clib(old_build_clib):
 
         self.build_libraries(self.libraries)
 
+        if self.inplace:
+            for l in  self.distribution.installed_libraries:
+                libname = self.compiler.library_filename(l.name) 
+                source = os.path.join(self.build_clib, libname)
+                target =  os.path.join(l.target_dir, libname)
+                self.mkpath(l.target_dir)
+                shutil.copy(source, target)
+
     def get_source_files(self):
         self.check_library_list(self.libraries)
         filenames = []
@@ -110,10 +125,9 @@ class build_clib(old_build_clib):
 
         sources = build_info.get('sources')
         if sources is None or not is_sequence(sources):
-            raise DistutilsSetupError, \
-                  ("in 'libraries' option (library '%s'), " +
+            raise DistutilsSetupError(("in 'libraries' option (library '%s'), " +
                    "'sources' must be present and must be " +
-                   "a list of source filenames") % lib_name
+                   "a list of source filenames") % lib_name)
         sources = list(sources)
 
         c_sources, cxx_sources, f_sources, fmodule_sources \
@@ -158,8 +172,8 @@ class build_clib(old_build_clib):
 
         # check availability of Fortran compilers
         if (f_sources or fmodule_sources) and fcompiler is None:
-            raise DistutilsError, "library %s has Fortran sources"\
-                  " but no Fortran compiler found" % (lib_name)
+            raise DistutilsError("library %s has Fortran sources"\
+                  " but no Fortran compiler found" % (lib_name))
 
         macros = build_info.get('macros')
         include_dirs = build_info.get('include_dirs')
