@@ -4,27 +4,48 @@
 import os
 import re
 import sys
+import shlex
+import copy
 
 from distutils.command import build_ext
 from distutils.dep_util import newer_group, newer
 from distutils.util import get_platform
 from distutils.errors import DistutilsError, DistutilsSetupError
 
-try:
-    import Pyrex.Compiler.Main
-    have_pyrex = True
-except ImportError:
-    have_pyrex = False
+def have_pyrex():
+    try:
+        import Pyrex.Compiler.Main
+        return True
+    except ImportError:
+        return False
 
 # this import can't be done here, as it uses numpy stuff only available
 # after it's installed
 #import numpy.f2py
 from numpy.distutils import log
 from numpy.distutils.misc_util import fortran_ext_match, \
-     appendpath, is_string, is_sequence
+     appendpath, is_string, is_sequence, get_cmd
 from numpy.distutils.from_template import process_file as process_f_file
 from numpy.distutils.conv_template import process_file as process_c_file
-from numpy.distutils.exec_command import splitcmdline
+
+def subst_vars(target, source, d):
+    """Substitute any occurence of @foo@ by d['foo'] from source file into
+    target."""
+    var = re.compile('@([a-zA-Z_]+)@')
+    fs = open(source, 'r')
+    try:
+        ft = open(target, 'w')
+        try:
+            for l in fs.readlines():
+                m = var.search(l)
+                if m:
+                    ft.write(l.replace('@%s@' % m.group(1), d[m.group(1)]))
+                else:
+                    ft.write(l)
+        finally:
+            ft.close()
+    finally:
+        fs.close()
 
 class build_src(build_ext.build_ext):
 
@@ -94,7 +115,7 @@ class build_src(build_ext.build_ext):
         if self.f2py_opts is None:
             self.f2py_opts = []
         else:
-            self.f2py_opts = splitcmdline(self.f2py_opts)
+            self.f2py_opts = shlex.split(self.f2py_opts)
 
         if self.swigflags:
             if self.swig_opts:
@@ -106,7 +127,7 @@ class build_src(build_ext.build_ext):
         if self.swig_opts is None:
             self.swig_opts = []
         else:
-            self.swig_opts = splitcmdline(self.swig_opts)
+            self.swig_opts = shlex.split(self.swig_opts)
 
         # use options from build_ext command
         build_ext = self.get_finalized_command('build_ext')
@@ -125,6 +146,7 @@ class build_src(build_ext.build_ext):
                     setattr(self, c, v)
 
     def run(self):
+        log.info("build_src")
         if not (self.extensions or self.libraries):
             return
         self.build_sources()
@@ -147,6 +169,7 @@ class build_src(build_ext.build_ext):
                 self.build_extension_sources(ext)
 
         self.build_data_files_sources()
+        self.build_npy_pkg_config()
 
     def build_data_files_sources(self):
         if not self.data_files:
@@ -163,8 +186,8 @@ class build_src(build_ext.build_ext):
                     build_dir = self.get_package_dir('.'.join(d.split(os.sep)))
                 else:
                     build_dir = os.path.join(self.build_src,d)
-                funcs = filter(callable,files)
-                files = filter(lambda f:not callable(f), files)
+                funcs = filter(lambda f:hasattr(f, '__call__'), files)
+                files = filter(lambda f:not hasattr(f, '__call__'), files)
                 for f in funcs:
                     if f.func_code.co_argcount==1:
                         s = f(build_dir)
@@ -183,6 +206,61 @@ class build_src(build_ext.build_ext):
                 raise TypeError(repr(data))
         self.data_files[:] = new_data_files
 
+
+    def _build_npy_pkg_config(self, info, gd):
+        import shutil
+        template, install_dir, subst_dict = info
+        template_dir = os.path.dirname(template)
+        for k, v in gd.items():
+            subst_dict[k] = v
+
+        if self.inplace == 1:
+            generated_dir = os.path.join(template_dir, install_dir)
+        else:
+            generated_dir = os.path.join(self.build_src, template_dir,
+                    install_dir)
+        generated = os.path.basename(os.path.splitext(template)[0])
+        generated_path = os.path.join(generated_dir, generated)
+        if not os.path.exists(generated_dir):
+            os.makedirs(generated_dir)
+
+        subst_vars(generated_path, template, subst_dict)
+
+        # Where to install relatively to install prefix
+        full_install_dir = os.path.join(template_dir, install_dir)
+        return full_install_dir, generated_path
+
+    def build_npy_pkg_config(self):
+        log.info('build_src: building npy-pkg config files')
+
+        # XXX: another ugly workaround to circumvent distutils brain damage. We
+        # need the install prefix here, but finalizing the options of the
+        # install command when only building sources cause error. Instead, we
+        # copy the install command instance, and finalize the copy so that it
+        # does not disrupt how distutils want to do things when with the
+        # original install command instance.
+        install_cmd = copy.copy(get_cmd('install'))
+        if not install_cmd.finalized == 1:
+            install_cmd.finalize_options()
+        build_npkg = False
+        gd = {}
+        if self.inplace == 1:
+            top_prefix = '.'
+            build_npkg = True
+        elif hasattr(install_cmd, 'install_libbase'):
+            top_prefix = install_cmd.install_libbase
+            build_npkg = True
+
+        if build_npkg:
+            for pkg, infos in self.distribution.installed_pkg_config.items():
+                pkg_path = self.distribution.package_dir[pkg]
+                prefix = os.path.join(os.path.abspath(top_prefix), pkg_path)
+                d = {'prefix': prefix}
+                for info in infos:
+                    install_dir, generated = self._build_npy_pkg_config(info, d)
+                    self.distribution.data_files.append((install_dir,
+                        [generated]))
+
     def build_py_modules_sources(self):
         if not self.py_modules:
             return
@@ -196,13 +274,13 @@ class build_src(build_ext.build_ext):
                 else:
                     build_dir = os.path.join(self.build_src,
                                              os.path.join(*package.split('.')))
-                if callable(source):
+                if hasattr(source, '__call__'):
                     target = os.path.join(build_dir, module_base + '.py')
                     source = source(target)
                 if source is None:
                     continue
                 modules = [(package, module_base, source)]
-                if not self.py_modules_dict.has_key(package):
+                if package not in self.py_modules_dict:
                     self.py_modules_dict[package] = []
                 self.py_modules_dict[package] += modules
             else:
@@ -259,7 +337,7 @@ class build_src(build_ext.build_ext):
 
         sources, py_files = self.filter_py_files(sources)
 
-        if not self.py_modules_dict.has_key(package):
+        if package not in self.py_modules_dict:
             self.py_modules_dict[package] = []
         modules = []
         for f in py_files:
@@ -292,7 +370,7 @@ class build_src(build_ext.build_ext):
         else:
             if is_sequence(extension):
                 name = extension[0]
-            #    if not extension[1].has_key('include_dirs'):
+            #    if 'include_dirs' not in extension[1]:
             #        extension[1]['include_dirs'] = []
             #    incl_dirs = extension[1]['include_dirs']
             else:
@@ -302,6 +380,7 @@ class build_src(build_ext.build_ext):
             #    incl_dirs.append(self.build_src)
             build_dir = os.path.join(*([self.build_src]\
                                        +name.split('.')[:-1]))
+        build_dir = os.path.normpath(build_dir)
         self.mkpath(build_dir)
         for func in func_sources:
             source = func(extension, build_dir)
@@ -385,14 +464,15 @@ class build_src(build_ext.build_ext):
         return new_sources
 
     def generate_a_pyrex_source(self, base, ext_name, source, extension):
-        if self.inplace or not have_pyrex:
+        if self.inplace or not have_pyrex():
             target_dir = os.path.dirname(base)
         else:
             target_dir = appendpath(self.build_src, os.path.dirname(base))
         target_file = os.path.join(target_dir, ext_name + '.c')
         depends = [source] + extension.depends
         if self.force or newer_group(depends, target_file, 'newer'):
-            if have_pyrex:
+            if have_pyrex():
+                import Pyrex.Compiler.Main
                 log.info("pyrexc:> %s" % (target_file))
                 self.mkpath(target_dir)
                 options = Pyrex.Compiler.Main.CompilationOptions(
@@ -402,8 +482,8 @@ class build_src(build_ext.build_ext):
                 pyrex_result = Pyrex.Compiler.Main.compile(source,
                                                            options=options)
                 if pyrex_result.num_errors != 0:
-                    raise DistutilsError,"%d errors while compiling %r with Pyrex" \
-                          % (pyrex_result.num_errors, source)
+                    raise DistutilsError("%d errors while compiling %r with Pyrex" \
+                          % (pyrex_result.num_errors, source))
             elif os.path.isfile(target_file):
                 log.warn("Pyrex required for compiling %r but not available,"\
                          " using old target %r"\
@@ -465,7 +545,8 @@ class build_src(build_ext.build_ext):
         if not (f2py_sources or f_sources):
             return new_sources
 
-        map(self.mkpath, target_dirs)
+        for d in target_dirs:
+            self.mkpath(d)
 
         f2py_options = extension.f2py_options + self.f2py_opts
 
@@ -560,7 +641,11 @@ class build_src(build_ext.build_ext):
         target_dirs = []
         py_files = []     # swig generated .py files
         target_ext = '.c'
-        if self.swig_cpp:
+        if '-c++' in extension.swig_opts:
+            typ = 'c++'
+            is_cpp = True
+            extension.swig_opts.remove('-c++')
+        elif self.swig_cpp:
             typ = 'c++'
             is_cpp = True
         else:
@@ -572,6 +657,8 @@ class build_src(build_ext.build_ext):
         for source in sources:
             (base, ext) = os.path.splitext(source)
             if ext == '.i': # SWIG interface file
+                # the code below assumes that the sources list
+                # contains not more than one .i SWIG interface file
                 if self.inplace:
                     target_dir = os.path.dirname(base)
                     py_target_dir = self.ext_target_dir
@@ -587,10 +674,16 @@ class build_src(build_ext.build_ext):
                     if typ is None:
                         typ = get_swig_target(source)
                         is_cpp = typ=='c++'
-                        if is_cpp: target_ext = '.cpp'
+                        if is_cpp: 
+                            target_ext = '.cpp'
                     else:
                         typ2 = get_swig_target(source)
-                        if typ!=typ2:
+                        if typ2 is None:
+                            log.warn('source %r does not define swig target, assuming %s swig target' \
+                                     % (source, typ))
+                            if is_cpp: 
+                                target_ext = '.cpp'
+                        elif typ!=typ2:
                             log.warn('expected %r but source %r defines %r swig target' \
                                      % (typ, source, typ2))
                             if typ2=='c++':
@@ -632,9 +725,11 @@ class build_src(build_ext.build_ext):
         if skip_swig:
             return new_sources + py_files
 
-        map(self.mkpath, target_dirs)
+        for d in target_dirs:
+            self.mkpath(d)
+
         swig = self.swig or self.find_swig()
-        swig_cmd = [swig, "-python"]
+        swig_cmd = [swig, "-python"] + extension.swig_opts
         if is_cpp:
             swig_cmd.append('-c++')
         for d in extension.include_dirs:
@@ -664,7 +759,7 @@ _has_cpp_header = re.compile(r'-[*]-\s*c[+][+]\s*-[*]-',re.I).search
 
 def get_swig_target(source):
     f = open(source,'r')
-    result = 'c'
+    result = None
     line = f.readline()
     if _has_cpp_header(line):
         result = 'c++'
