@@ -2,22 +2,29 @@ import re
 import os
 import sys
 import warnings
+import platform
+import tempfile
+from subprocess import Popen, PIPE, STDOUT
 
 from numpy.distutils.cpuinfo import cpu
 from numpy.distutils.fcompiler import FCompiler
 from numpy.distutils.exec_command import exec_command
 from numpy.distutils.misc_util import msvc_runtime_library
+from numpy.distutils.compat import get_exception
 
 compilers = ['GnuFCompiler', 'Gnu95FCompiler']
 
 TARGET_R = re.compile("Target: ([a-zA-Z0-9_\-]*)")
 
-# XXX: do we really need to check for target ? If the arch is not supported,
-# the return code should be != 0
-_R_ARCHS = {"ppc": r"^Target: (powerpc-.*)$",
-    "i686": r"^Target: (i686-.*)$",
-    "x86_64": r"^Target: (i686-.*)$",
-    "ppc64": r"^Target: (powerpc-.*)$",}
+# XXX: handle cross compilation
+def is_win64():
+    return sys.platform == "win32" and platform.architecture()[0] == "64bit"
+
+if is_win64():
+    #_EXTRAFLAGS = ["-fno-leading-underscore"]
+    _EXTRAFLAGS = []
+else:
+    _EXTRAFLAGS = []
 
 class GnuFCompiler(FCompiler):
     compiler_type = 'gnu'
@@ -97,14 +104,14 @@ class GnuFCompiler(FCompiler):
         if sys.platform=='darwin':
             target = os.environ.get('MACOSX_DEPLOYMENT_TARGET', None)
             # If MACOSX_DEPLOYMENT_TARGET is set, we simply trust the value
-            # and leave it alone.  But, distutils will complain if the 
-            # environment's value is different from the one in the Python 
-            # Makefile used to build Python.  We let disutils handle this 
+            # and leave it alone.  But, distutils will complain if the
+            # environment's value is different from the one in the Python
+            # Makefile used to build Python.  We let disutils handle this
             # error checking.
             if not target:
-                # If MACOSX_DEPLOYMENT_TARGET is not set in the environment, 
-                # we try to get it first from the Python Makefile and then we 
-                # fall back to setting it to 10.3 to maximize the set of 
+                # If MACOSX_DEPLOYMENT_TARGET is not set in the environment,
+                # we try to get it first from the Python Makefile and then we
+                # fall back to setting it to 10.3 to maximize the set of
                 # versions we can work with.  This is a reasonable default
                 # even when using the official Python dist and those derived
                 # from it.
@@ -117,7 +124,7 @@ class GnuFCompiler(FCompiler):
                 if target == '10.3':
                     s = 'Env. variable MACOSX_DEPLOYMENT_TARGET set to 10.3'
                     warnings.warn(s)
-            
+
             opt.extend(['-undefined', 'dynamic_lookup', '-bundle'])
         else:
             opt.append("-shared")
@@ -185,15 +192,30 @@ class GnuFCompiler(FCompiler):
         return ['-g']
 
     def get_flags_opt(self):
-        if self.get_version()<='3.3.3':
+        v = self.get_version()
+        if v and v<='3.3.3':
             # With this compiler version building Fortran BLAS/LAPACK
             # with -O3 caused failures in lib.lapack heevr,syevr tests.
             opt = ['-O2']
         else:
             opt = ['-O3']
-        opt.append('-funroll-loops')
+# Segmentation fault on powerpc when building py-scipy
+#        opt.append('-funroll-loops')
         return opt
 
+    def _c_arch_flags(self):
+        """ Return detected arch flags from CFLAGS """
+        from distutils import sysconfig
+        try:
+            cflags = sysconfig.get_config_vars()['CFLAGS']
+        except KeyError:
+            return []
+        arch_re = re.compile(r"-arch\s+(\w+)")
+        arch_flags = []
+        for arch in arch_re.findall(cflags):
+            arch_flags += ['-arch', arch]
+        return arch_flags
+
     def get_flags_arch(self):
         return []
 
@@ -206,7 +228,17 @@ class Gnu95FCompiler(GnuFCompiler):
         v = self.gnu_version_match(version_string)
         if not v or v[0] != 'gfortran':
             return None
-        return v[1]
+        v = v[1]
+        if v>='4.':
+            # gcc-4 series releases do not support -mno-cygwin option
+            pass
+        else:
+            # use -mno-cygwin flag for gfortran when Python is not Cygwin-Python
+            if sys.platform == 'win32':
+                for key in ['version_cmd', 'compiler_f77', 'compiler_f90',
+                            'compiler_fix', 'linker_so', 'linker_exe']:
+                    self.executables[key].append('-mno-cygwin')
+        return v
 
     # 'gfortran --version' results:
     # XXX is the below right?
@@ -220,50 +252,34 @@ class Gnu95FCompiler(GnuFCompiler):
     executables = {
         'version_cmd'  : ["<F90>", "--version"],
         'compiler_f77' : [None, "-Wall", "-ffixed-form",
-                          "-fno-second-underscore"],
-        'compiler_f90' : [None, "-Wall", "-fno-second-underscore"],
+		"-fno-second-underscore"] + _EXTRAFLAGS,
+        'compiler_f90' : [None, "-Wall", "-fno-second-underscore"] + _EXTRAFLAGS,
         'compiler_fix' : [None, "-Wall", "-ffixed-form",
-                          "-fno-second-underscore"],
+                          "-fno-second-underscore"] + _EXTRAFLAGS,
         'linker_so'    : ["<F90>", "-Wall"],
         'archiver'     : ["ar", "-cr"],
         'ranlib'       : ["ranlib"],
         'linker_exe'   : [None, "-Wall"]
         }
 
-    # use -mno-cygwin flag for g77 when Python is not Cygwin-Python
-    if sys.platform == 'win32':
-        for key in ['version_cmd', 'compiler_f77', 'compiler_f90',
-                    'compiler_fix', 'linker_so', 'linker_exe']:
-            executables[key].append('-mno-cygwin')
-
     module_dir_switch = '-J'
     module_include_switch = '-I'
 
     g2c = 'gfortran'
 
-    # Note that this is here instead of GnuFCompiler as gcc < 4 uses a
-    # different output format (which isn't as useful) than gcc >= 4,
-    # and we don't have to worry about g77 being universal (as it can't be).
-    def _can_target(self, cmd, arch):
-        """Return true is the compiler support the -arch flag for the given
-        architecture."""
-        newcmd = cmd[:]
-        newcmd.extend(["-arch", arch, "-v"])
-        st, out = exec_command(" ".join(newcmd))
-        if st == 0:
-            for line in out.splitlines():
-                m = re.search(_R_ARCHS[arch], line)
-                if m:
-                    return True
-        return False
-            
     def _universal_flags(self, cmd):
         """Return a list of -arch flags for every supported architecture."""
         if not sys.platform == 'darwin':
             return []
         arch_flags = []
-        for arch in ["ppc", "i686"]:
-            if self._can_target(cmd, arch):
+        # get arches the C compiler gets.
+        c_archs = self._c_arch_flags()
+        if "i386" in c_archs:
+            c_archs[c_archs.index("i386")] = "i686"
+        # check the arches the Fortran compiler supports, and compare with
+        # arch flags from C compiler
+        for arch in ["ppc", "i686", "x86_64", "ppc64"]:
+            if _can_target(cmd, arch) and arch in c_archs:
                 arch_flags.extend(["-arch", arch])
         return arch_flags
 
@@ -307,6 +323,13 @@ class Gnu95FCompiler(GnuFCompiler):
                     i = opt.index("gcc")
                     opt.insert(i+1, "mingwex")
                     opt.insert(i+1, "mingw32")
+            # XXX: fix this mess, does not work for mingw
+            if is_win64():
+                c_compiler = self.c_compiler
+                if c_compiler and c_compiler.compiler_type == "msvc":
+                    return []
+                else:
+                    raise NotImplementedError("Only MS compiler supported with gfortran on win64")
         return opt
 
     def get_target(self):
@@ -319,17 +342,44 @@ class Gnu95FCompiler(GnuFCompiler):
                 return m.group(1)
         return ""
 
+    def get_flags_opt(self):
+        if is_win64():
+            return ['-O0']
+        else:
+            return GnuFCompiler.get_flags_opt(self)
+
+def _can_target(cmd, arch):
+    """Return true is the command supports the -arch flag for the given
+    architecture."""
+    newcmd = cmd[:]
+    fid, filename = tempfile.mkstemp(suffix=".f")
+    try:
+        d = os.path.dirname(filename)
+        output = os.path.splitext(filename)[0] + ".o"
+        try:
+            newcmd.extend(["-arch", arch, "-c", filename])
+            p = Popen(newcmd, stderr=STDOUT, stdout=PIPE, cwd=d)
+            p.communicate()
+            return p.returncode == 0
+        finally:
+            if os.path.exists(output):
+                os.remove(output)
+    finally:
+        os.remove(filename)
+    return False
+
 if __name__ == '__main__':
     from distutils import log
     log.set_verbosity(2)
     compiler = GnuFCompiler()
     compiler.customize()
-    print compiler.get_version()
+    print(compiler.get_version())
     raw_input('Press ENTER to continue...')
     try:
         compiler = Gnu95FCompiler()
         compiler.customize()
-        print compiler.get_version()
-    except Exception, msg:
-        print msg
+        print(compiler.get_version())
+    except Exception:
+        msg = get_exception()
+        print(msg)
     raw_input('Press ENTER to continue...')
