@@ -21,12 +21,15 @@ class HPUXFCompiler(FCompiler):
     module_include_switch = None #XXX: fix me
     pic_flags = ['+pic=long']
     def get_flags(self):
-        return self.pic_flags + ['+ppu']
+        return self.pic_flags + ['+ppu', '+DD64']
     def get_flags_opt(self):
         return ['-O3']
     def get_libraries(self):
         return ['m']
-    def get_version(self, force=0, ok_status=[256,0]):
+    def get_library_dirs(self):
+        opt = ['/usr/lib/hpux64']
+        return opt
+    def get_version(self, force=0, ok_status=[256,0,1]):
         # XXX status==256 may indicate 'unrecognized option' or
         #     'no input file'. So, version_cmd needs more work.
         return FCompiler.get_version(self,force,ok_status)
@@ -37,4 +40,4 @@ if __name__ == '__main__':
     from numpy.distutils.fcompiler import new_fcompiler
     compiler = new_fcompiler(compiler='hpux')
     compiler.customize()
-    print compiler.get_version()
+    print(compiler.get_version())
