@@ -11,13 +11,16 @@ Support code for building Python extensions on Windows.
 import os
 import subprocess
 import sys
-import log
 import subprocess
 import re
 
 # Overwrite certain distutils.ccompiler functions:
 import numpy.distutils.ccompiler
 
+if sys.version_info[0] < 3:
+    import log
+else:
+    from numpy.distutils import log
 # NT stuff
 # 1. Make sure libpython<version>.a exists for gcc.  If not, build it.
 # 2. Force windows to use gcc (we're struggling with MSVC and g77 support)
@@ -101,11 +104,19 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
         # bad consequences, like using Py_ModuleInit4 instead of
         # Py_ModuleInit4_64, etc... So we add it here
         if get_build_architecture() == 'AMD64':
-            self.set_executables(
-                    compiler='gcc -DMS_WIN64 -mno-cygwin -O0 -Wall',
-                    compiler_so='gcc -DMS_WIN64 -mno-cygwin -O0 -Wall -Wstrict-prototypes',
-                    linker_exe='gcc -mno-cygwin',
-                    linker_so='gcc -mno-cygwin -shared')
+            if self.gcc_version < "4.":
+                self.set_executables(
+                    compiler='gcc -g -DDEBUG -DMS_WIN64 -mno-cygwin -O0 -Wall',
+                    compiler_so='gcc -g -DDEBUG -DMS_WIN64 -mno-cygwin -O0 -Wall -Wstrict-prototypes',
+                    linker_exe='gcc -g -mno-cygwin',
+                    linker_so='gcc -g -mno-cygwin -shared')
+            else:
+                # gcc-4 series releases do not support -mno-cygwin option
+                self.set_executables(
+                    compiler='gcc -g -DDEBUG -DMS_WIN64 -O0 -Wall',
+                    compiler_so='gcc -g -DDEBUG -DMS_WIN64 -O0 -Wall -Wstrict-prototypes',
+                    linker_exe='gcc -g',
+                    linker_so='gcc -g -shared')
         else:
             if self.gcc_version <= "3.0.0":
                 self.set_executables(compiler='gcc -mno-cygwin -O2 -w',
@@ -113,11 +124,17 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
                                      linker_exe='g++ -mno-cygwin',
                                      linker_so='%s -mno-cygwin -mdll -static %s'
                                      % (self.linker, entry_point))
-            else:
+            elif self.gcc_version < "4.":
                 self.set_executables(compiler='gcc -mno-cygwin -O2 -Wall',
                                      compiler_so='gcc -mno-cygwin -O2 -Wall -Wstrict-prototypes',
                                      linker_exe='g++ -mno-cygwin',
                                      linker_so='g++ -mno-cygwin -shared')
+            else:
+                # gcc-4 series releases do not support -mno-cygwin option
+                self.set_executables(compiler='gcc -O2 -Wall',
+                                     compiler_so='gcc -O2 -Wall -Wstrict-prototypes',
+                                     linker_exe='g++ ',
+                                     linker_so='g++ -shared')
         # added for python2.3 support
         # we can't pass it through set_executables because pre 2.2 would fail
         self.compiler_cxx = ['g++']
@@ -171,7 +188,10 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
             func = distutils.cygwinccompiler.CygwinCCompiler.link
         else:
             func = UnixCCompiler.link
-        func(*args[:func.im_func.func_code.co_argcount])
+        if sys.version_info[0] >= 3:
+            func(*args[:func.__code__.co_argcount])
+        else:
+            func(*args[:func.im_func.func_code.co_argcount])
         return
 
     def object_filenames (self,
@@ -192,9 +212,9 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
                 base = base[1:]
 
             if ext not in (self.src_extensions + ['.rc','.res']):
-                raise UnknownFileError, \
+                raise UnknownFileError(
                       "unknown file type '%s' (from '%s')" % \
-                      (ext, src_name)
+                      (ext, src_name))
             if strip_dir:
                 base = os.path.basename (base)
             if ext == '.res' or ext == '.rc':
@@ -212,7 +232,7 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
 def find_python_dll():
     maj, min, micro = [int(i) for i in sys.version_info[:3]]
     dllname = 'python%d%d.dll' % (maj, min)
-    print "Looking for %s" % dllname
+    print ("Looking for %s" % dllname)
 
     # We can't do much here:
     # - find it in python main dir
@@ -433,10 +453,10 @@ def check_embedded_msvcr_match_linked(msver):
     if msvcv:
         maj = int(msvcv[5:6])
         if not maj == int(msver):
-            raise ValueError, \
+            raise ValueError(
                   "Discrepancy between linked msvcr " \
                   "(%d) and the one about to be embedded " \
-                  "(%d)" % (int(msver), maj)
+                  "(%d)" % (int(msver), maj))
 
 def configtest_name(config):
     base = os.path.basename(config._gen_temp_sourcefile("yo", [], "c"))
