@@ -116,7 +116,10 @@ import re
 import copy
 import warnings
 from glob import glob
-import ConfigParser
+if sys.version_info[0] < 3:
+    from ConfigParser import NoOptionError, ConfigParser
+else:
+    from configparser import NoOptionError, ConfigParser
 
 from distutils.errors import DistutilsError
 from distutils.dist import Distribution
@@ -127,6 +130,10 @@ from numpy.distutils.exec_command import \
     find_executable, exec_command, get_pythonexe
 from numpy.distutils.misc_util import is_sequence, is_string
 from numpy.distutils.command.config import config as cmd_config
+from numpy.distutils.compat import get_exception
+
+if sys.version_info[0] >= 3:
+    from functools import reduce
 
 # Determine number of bits
 import platform
@@ -149,11 +156,11 @@ def libpaths(paths,bits):
     >>> paths = ['/usr/X11R6/lib','/usr/X11/lib','/usr/lib']
 
     For a 32-bit platform, this is already valid:
-    >>> libpaths(paths,32)
+    >>> np.distutils.system_info.libpaths(paths,32)
     ['/usr/X11R6/lib', '/usr/X11/lib', '/usr/lib']
 
     On 64 bits, we prepend the '64' postfix
-    >>> libpaths(paths,64)
+    >>> np.distutils.system_info.libpaths(paths,64)
     ['/usr/X11R6/lib64', '/usr/X11R6/lib', '/usr/X11/lib64', '/usr/X11/lib',
     '/usr/lib64', '/usr/lib']
     """
@@ -204,6 +211,10 @@ default_include_dirs = filter(os.path.isdir, default_include_dirs)
 default_src_dirs = filter(os.path.isdir, default_src_dirs)
 
 so_ext = distutils.sysconfig.get_config_vars('SO')[0] or ''
+# fix long extension for Python >=3.2, see PEP 3149.
+if 'SOABI' in distutils.sysconfig.get_config_vars():
+    so_ext = so_ext.replace('.'+distutils.sysconfig.get_config_var('SOABI'), '', 1)
+
 
 def get_standard_file(fname):
     """Returns a list of files named 'fname' from
@@ -395,7 +406,7 @@ class system_info:
         defaults['include_dirs'] = os.pathsep.join(default_include_dirs)
         defaults['src_dirs'] = os.pathsep.join(default_src_dirs)
         defaults['search_static_first'] = str(self.search_static_first)
-        self.cp = ConfigParser.ConfigParser(defaults)
+        self.cp = ConfigParser(defaults)
         self.files = []
         self.files.extend(get_standard_file('.numpy-site.cfg'))
         self.files.extend(get_standard_file('site.cfg'))
@@ -451,7 +462,7 @@ class system_info:
                     if notfound_action==1:
                         warnings.warn(self.notfounderror.__doc__)
                     elif notfound_action==2:
-                        raise self.notfounderror,self.notfounderror.__doc__
+                        raise self.notfounderror(self.notfounderror.__doc__)
                     else:
                         raise ValueError(repr(notfound_action))
 
@@ -516,8 +527,13 @@ class system_info:
         dirs.extend(default_dirs)
         ret = []
         for d in dirs:
-            if os.path.isdir(d) and d not in ret:
+            if not os.path.isdir(d):
+                warnings.warn('Specified path %s is invalid.' % d)
+                continue
+
+            if d not in ret:
                 ret.append(d)
+
         log.debug('( %s = %s )', key, ':'.join(ret))
         return ret
 
@@ -533,7 +549,7 @@ class system_info:
     def get_libs(self, key, default):
         try:
             libs = self.cp.get(self.section, key)
-        except ConfigParser.NoOptionError:
+        except NoOptionError:
             if not default:
                 return []
             if is_string(default):
@@ -999,6 +1015,9 @@ class atlas_info(system_info):
             else:
                 info['language'] = 'f77'
 
+        atlas_version, atlas_extra_info = get_atlas_version(**atlas)
+        dict_append(info, **atlas_extra_info)
+
         self.set_info(**info)
 
 class atlas_blas_info(atlas_info):
@@ -1023,6 +1042,9 @@ class atlas_blas_info(atlas_info):
             dict_append(info,include_dirs=[h])
         info['language'] = 'c'
 
+        atlas_version, atlas_extra_info = get_atlas_version(**atlas)
+        dict_append(atlas, **atlas_extra_info)
+
         dict_append(info,**atlas)
 
         self.set_info(**info)
@@ -1168,6 +1190,10 @@ class lapack_src_info(system_info):
         # Lapack 3.1:
         src_dir2 = os.path.join(src_dir,'..','INSTALL')
         sources += [os.path.join(src_dir2,p+'lamch.f') for p in 'sdcz']
+        # Lapack 3.2.1:
+        sources += [os.path.join(src_dir,p+'larfp.f') for p in 'sdcz']
+        sources += [os.path.join(src_dir,'ila'+p+'lr.f') for p in 'sdcz']
+        sources += [os.path.join(src_dir,'ila'+p+'lc.f') for p in 'sdcz']
         # Should we check here actual existence of source files?
         # Yes, the file listing is different between 3.0 and 3.1
         # versions.
@@ -1193,22 +1219,44 @@ def get_atlas_version(**config):
         return _cached_atlas_version[key]
     c = cmd_config(Distribution())
     atlas_version = None
+    info = {}
     try:
         s, o = c.get_output(atlas_version_c_text,
                             libraries=libraries, library_dirs=library_dirs)
-    except: # failed to get version from file -- maybe on Windows
+        if s and re.search (r'undefined reference to `_gfortran', o, re.M):
+            s, o = c.get_output(atlas_version_c_text,
+                                libraries=libraries + ['gfortran'], library_dirs=library_dirs)
+            if not s:
+                warnings.warn("""
+*****************************************************
+Linkage with ATLAS requires gfortran. Use
+
+  python setup.py config_fc --fcompiler=gnu95 ...
+
+when building extension libraries that use ATLAS.
+Make sure that -lgfortran is used for C++ extensions.
+*****************************************************
+""")
+                dict_append(info, language='f90',
+                            define_macros = [('ATLAS_REQUIRES_GFORTRAN', None)])
+    except Exception, msg: # failed to get version from file -- maybe on Windows
         # look at directory name
         for o in library_dirs:
             m = re.search(r'ATLAS_(?P<version>\d+[.]\d+[.]\d+)_',o)
             if m:
                 atlas_version = m.group('version')
             if atlas_version is not None:
+
                 break
         # final choice --- look at ATLAS_VERSION environment
         #   variable
         if atlas_version is None:
             atlas_version = os.environ.get('ATLAS_VERSION',None)
-        return atlas_version or '?.?.?'
+        if atlas_version:
+            dict_append(info, define_macros = [('ATLAS_INFO','"\\"%s\\""' % atlas_version)])
+        else:
+            dict_append(info, define_macros = [('NO_ATLAS_INFO',-1)])
+        return atlas_version or '?.?.?', info
 
     if not s:
         m = re.search(r'ATLAS version (?P<version>\d+[.]\d+[.]\d+)',o)
@@ -1220,8 +1268,13 @@ def get_atlas_version(**config):
         else:
             log.info('Status: %d', s)
             log.info('Output: %s', o)
-    _cached_atlas_version[key] = atlas_version
-    return atlas_version
+
+    if atlas_version=='3.2.1_pre3.3.6':
+        dict_append(info, define_macros = [('NO_ATLAS_INFO',-2)])
+    else:
+        dict_append(info, define_macros = [('ATLAS_INFO','"\\"%s\\""' % atlas_version)])
+    result = _cached_atlas_version[key] = atlas_version, info
+    return result
 
 from distutils.util import get_platform
 
@@ -1270,22 +1323,12 @@ class lapack_opt_info(system_info):
         need_blas = 0
         info = {}
         if atlas_info:
-            version_info = atlas_info.copy()
-            atlas_version = get_atlas_version(**version_info)
-            if 'define_macros' not in atlas_info:
-                atlas_info['define_macros'] = []
-            if atlas_version is None:
-                atlas_info['define_macros'].append(('NO_ATLAS_INFO',2))
-            else:
-                atlas_info['define_macros'].append(('ATLAS_INFO',
-                                                    '"\\"%s\\""' % atlas_version))
-            if atlas_version=='3.2.1_pre3.3.6':
-                atlas_info['define_macros'].append(('NO_ATLAS_INFO',4))
-            l = atlas_info.get('define_macros',[])
+            l = atlas_info.get ('define_macros', [])
             if ('ATLAS_WITH_LAPACK_ATLAS',None) in l \
                    or ('ATLAS_WITHOUT_LAPACK',None) in l:
                 need_lapack = 1
             info = atlas_info
+
         else:
             warnings.warn(AtlasNotFoundError.__doc__)
             need_blas = 1
@@ -1369,15 +1412,6 @@ class blas_opt_info(system_info):
         need_blas = 0
         info = {}
         if atlas_info:
-            version_info = atlas_info.copy()
-            atlas_version = get_atlas_version(**version_info)
-            if 'define_macros' not in atlas_info:
-                atlas_info['define_macros'] = []
-            if atlas_version is None:
-                atlas_info['define_macros'].append(('NO_ATLAS_INFO',2))
-            else:
-                atlas_info['define_macros'].append(('ATLAS_INFO',
-                                                    '"\\"%s\\""' % atlas_version))
             info = atlas_info
         else:
             warnings.warn(AtlasNotFoundError.__doc__)
@@ -1516,6 +1550,13 @@ class _numpy_info(system_info):
                 if name=='lib':
                     break
                 prefix.append(name)
+
+            # Ask numpy for its own include path before attempting anything else
+            try:
+                include_dirs.append(getattr(module, 'get_include')())
+            except AttributeError:
+                pass
+
             include_dirs.append(distutils.sysconfig.get_python_inc(
                                         prefix=os.sep.join(prefix)))
         except ImportError:
@@ -1591,15 +1632,18 @@ class numerix_info(system_info):
             try:
                 import numpy
                 which = "numpy", "defaulted"
-            except ImportError,msg1:
+            except ImportError:
+                msg1 = str(get_exception())
                 try:
                     import Numeric
                     which = "numeric", "defaulted"
-                except ImportError,msg2:
+                except ImportError:
+                    msg2 = str(get_exception())
                     try:
                         import numarray
                         which = "numarray", "defaulted"
-                    except ImportError,msg3:
+                    except ImportError:
+                        msg3 = str(get_exception())
                         log.info(msg1)
                         log.info(msg2)
                         log.info(msg3)
