@@ -13,4 +13,4 @@ def configuration(parent_package='',top_path=None):
 
 if __name__ == "__main__":
     from numpy.distutils.core import setup
-    setup(**configuration(top_path='').todict())
+    setup(configuration=configuration)
