@@ -7,23 +7,52 @@ import os
 from distutils.errors import DistutilsExecError, CompileError
 from distutils.unixccompiler import *
 from numpy.distutils.ccompiler import replace_method
+from numpy.distutils.compat import get_exception
 
-import log
+if sys.version_info[0] < 3:
+    import log
+else:
+    from numpy.distutils import log
 
 # Note that UnixCCompiler._compile appeared in Python 2.3
 def UnixCCompiler__compile(self, obj, src, ext, cc_args, extra_postargs, pp_opts):
+    """Compile a single source files with a Unix-style compiler."""
     display = '%s: %s' % (os.path.basename(self.compiler_so[0]),src)
     try:
         self.spawn(self.compiler_so + cc_args + [src, '-o', obj] +
                    extra_postargs, display = display)
-    except DistutilsExecError, msg:
-        raise CompileError, msg
+    except DistutilsExecError:
+        msg = str(get_exception())
+        raise CompileError(msg)
 
 replace_method(UnixCCompiler, '_compile', UnixCCompiler__compile)
 
 
 def UnixCCompiler_create_static_lib(self, objects, output_libname,
                                     output_dir=None, debug=0, target_lang=None):
+    """
+    Build a static library in a separate sub-process.
+
+    Parameters
+    ----------
+    objects : list or tuple of str
+        List of paths to object files used to build the static library.
+    output_libname : str
+        The library name as an absolute or relative (if `output_dir` is used)
+        path.
+    output_dir : str, optional
+        The path to the output directory. Default is None, in which case
+        the ``output_dir`` attribute of the UnixCCompiler instance.
+    debug : bool, optional
+        This parameter is not used.
+    target_lang : str, optional
+        This parameter is not used.
+
+    Returns
+    -------
+    None
+
+    """
     objects, output_dir = self._fix_object_args(objects, output_dir)
 
     output_filename = \
@@ -59,8 +88,9 @@ def UnixCCompiler_create_static_lib(self, objects, output_libname,
             try:
                 self.spawn(self.ranlib + [output_filename],
                            display = display)
-            except DistutilsExecError, msg:
-                raise LibError, msg
+            except DistutilsExecError:
+                msg = str(get_exception())
+                raise LibError(msg)
     else:
         log.debug("skipping %s (up-to-date)", output_filename)
     return
