@@ -19,6 +19,7 @@ f2py_version = __version__.version
 import copy
 import re
 import os
+import sys
 from auxfuncs import *
 from crackfortran import markoutercomma
 import cb_rules
@@ -137,6 +138,11 @@ c2buildvalue_map={'double':'d',
                   'complex_double':'N',
                   'complex_long_double':'N',
                   'string':'z'}
+
+if sys.version_info[0] >= 3:
+    # Bytes, not Unicode strings
+    c2buildvalue_map['string'] = 'y'
+
 if using_newcore:
     #c2buildvalue_map=???
     pass
@@ -186,8 +192,8 @@ if os.path.isfile('.f2py_f2cmap'):
                 else:
                     errmess("\tIgnoring map {'%s':{'%s':'%s'}}: '%s' must be in %s\n"%(k,k1,d[k][k1],d[k][k1],c2py_map.keys()))
         outmess('Succesfully applied user defined changes from .f2py_f2cmap\n')
-    except:
-        errmess('Failed to apply user defined changes from .f2py_f2cmap. Skipping.\n')
+    except Exception, msg:
+        errmess('Failed to apply user defined changes from .f2py_f2cmap: %s. Skipping.\n' % (msg))
 cformat_map={'double':'%g',
              'float':'%g',
              'long_double':'%Lg',
@@ -245,8 +251,9 @@ def getctype(var):
                     try:
                         ctype=f2cmap[str(var['kindselector']['kind'])]
                     except KeyError:
-                        errmess('getctype: "%s(kind=%s)" not supported (use .f2py_f2cmap).\n'\
-                                %(typespec,var['kindselector']['kind']))
+                        errmess('getctype: "%s(kind=%s)" is mapped to C "%s" (to override define dict(%s = dict(%s="<C typespec>")) in %s/.f2py_f2cmap file).\n'\
+                                %(typespec,var['kindselector']['kind'], ctype,
+                                  typespec,var['kindselector']['kind'], os.getcwd()))
 
     else:
         if not isexternal(var):
@@ -429,7 +436,8 @@ def getinit(a,var):
                 else:
                     v = eval(v,{},{})
                     ret['init.r'],ret['init.i']=str(v.real),str(v.imag)
-            except: raise 'sign2map: expected complex number `(r,i)\' but got `%s\' as initial value of %s.'%(init,`a`)
+            except:
+                raise ValueError('getinit: expected complex number `(r,i)\' but got `%s\' as initial value of %r.' % (init, a))
             if isarray(var):
                 init='(capi_c.r=%s,capi_c.i=%s,capi_c)'%(ret['init.r'],ret['init.i'])
         elif isstring(var):
@@ -636,10 +644,18 @@ def modsign2map(m):
     else:
         ret['interface_usercode'] = ''
     ret['pymethoddef'] = getpymethoddef(m) or ''
+    if 'coutput' in m:
+        ret['coutput'] = m['coutput']
+    if 'f2py_wrapper_output' in m:
+        ret['f2py_wrapper_output'] = m['f2py_wrapper_output']
     return ret
 
-def cb_sign2map(a,var):
+def cb_sign2map(a,var,index=None):
     ret={'varname':a}
+    if index is None or 1: # disable 7712 patch
+        ret['varname_i'] = ret['varname']
+    else:
+        ret['varname_i'] = ret['varname'] + '_' + str(index)
     ret['ctype']=getctype(var)
     if ret['ctype'] in c2capi_map:
         ret['atype']=c2capi_map[ret['ctype']]
