@@ -59,6 +59,7 @@ includes['arrayobject.h']='''#define PY_ARRAY_UNIQUE_SYMBOL PyArray_API
 #include "arrayobject.h"'''
 
 includes['arrayobject.h']='#include "fortranobject.h"'
+includes['stdarg.h']='#include <stdarg.h>'
 
 ############# Type definitions ###############
 
@@ -230,8 +231,12 @@ cppmacros['PRINTPYOBJERR']="""\
 \tfprintf(stderr,\"\\n\");
 """
 cppmacros['MINMAX']="""\
+#ifndef max
 #define max(a,b) ((a > b) ? (a) : (b))
+#endif
+#ifndef min
 #define min(a,b) ((a < b) ? (a) : (b))
+#endif
 #ifndef MAX
 #define MAX(a,b) ((a > b) ? (a) : (b))
 #endif
@@ -239,6 +244,7 @@ cppmacros['MINMAX']="""\
 #define MIN(a,b) ((a < b) ? (a) : (b))
 #endif
 """
+needs['len..']=['f2py_size']
 cppmacros['len..']="""\
 #define rank(var) var ## _Rank
 #define shape(var,dim) var ## _Dims[dim]
@@ -247,9 +253,36 @@ cppmacros['len..']="""\
 #define fshape(var,dim) shape(var,rank(var)-dim-1)
 #define len(var) shape(var,0)
 #define flen(var) fshape(var,0)
-#define size(var) PyArray_SIZE((PyArrayObject *)(capi_ ## var ## _tmp))
+#define old_size(var) PyArray_SIZE((PyArrayObject *)(capi_ ## var ## _tmp))
 /* #define index(i) capi_i ## i */
 #define slen(var) capi_ ## var ## _len
+#define size(var, dim...) f2py_size((PyArrayObject *)(capi_ ## var ## _tmp), ##dim, -1)
+"""
+needs['f2py_size']=['stdarg.h']
+cfuncs['f2py_size']="""\
+int f2py_size(PyArrayObject* var, ...)
+{
+  npy_int sz = 0;
+  npy_int dim;
+  npy_int rank;
+  va_list argp;
+  va_start(argp, var);
+  dim = va_arg(argp, npy_int);
+  if (dim==-1)
+    {
+      sz = PyArray_SIZE(var);
+    }
+  else
+    {
+      rank = PyArray_NDIM(var);
+      if (dim>=1 && dim<=rank)
+        sz = PyArray_DIM(var, dim-1);
+      else
+        fprintf(stderr, \"f2py_size: 2nd argument value=%d fails to satisfy 1<=value<=%d. Result will be 0.\\n\", dim, rank);
+    }
+  va_end(argp);
+  return sz;
+}
 """
 
 cppmacros['pyobj_from_char1']='#define pyobj_from_char1(v) (PyInt_FromLong(v))'
@@ -278,6 +311,8 @@ needs['pyobj_from_complex_float1']=['complex_float']
 cppmacros['pyobj_from_complex_float1']='#define pyobj_from_complex_float1(v) (PyComplex_FromDoubles(v.r,v.i))'
 needs['pyobj_from_string1']=['string']
 cppmacros['pyobj_from_string1']='#define pyobj_from_string1(v) (PyString_FromString((char *)v))'
+needs['pyobj_from_string1size']=['string']
+cppmacros['pyobj_from_string1size']='#define pyobj_from_string1size(v,len) (PyString_FromStringAndSize((char *)v, len))'
 needs['TRYPYARRAYTEMPLATE']=['PRINTPYOBJERR']
 cppmacros['TRYPYARRAYTEMPLATE']="""\
 /* New SciPy */
@@ -338,10 +373,7 @@ cppmacros['TRYCOMPLEXPYARRAYTEMPLATE']="""\
                 case PyArray_SHORT: *(short *)(arr->data)=(*v).r; break;\\
                 case PyArray_UBYTE: *(unsigned char *)(arr->data)=(*v).r; break;\\
                 case PyArray_BYTE: *(signed char *)(arr->data)=(*v).r; break;\\
-                case PyArray_BOOL: *(npy_bool *)(arr->data)=((*v).r!=0 && (*v).i!=0)); break;\\
-                case PyArray_UBYTE: *(unsigned char *)(arr->data)=(*v).r; break;\\
-                case PyArray_BYTE: *(signed char *)(arr->data)=(*v).r; break;\\
-                case PyArray_SHORT: *(short *)(arr->data)=(*v).r; break;\\
+                case PyArray_BOOL: *(npy_bool *)(arr->data)=((*v).r!=0 && (*v).i!=0); break;\\
                 case PyArray_USHORT: *(npy_ushort *)(arr->data)=(*v).r; break;\\
                 case PyArray_UINT: *(npy_uint *)(arr->data)=(*v).r; break;\\
                 case PyArray_ULONG: *(npy_ulong *)(arr->data)=(*v).r; break;\\
@@ -600,8 +632,26 @@ fprintf(stderr,\"string_from_pyobj(str='%s',len=%d,inistr='%s',obj=%p)\\n\",(cha
 \t\ttmp = obj;
 \t\tPy_INCREF(tmp);
 \t}
-\telse
+#if PY_VERSION_HEX >= 0x03000000
+\telse if (PyUnicode_Check(obj)) {
+\t\ttmp = PyUnicode_AsASCIIString(obj);
+\t}
+\telse {
+\t\tPyObject *tmp2;
+\t\ttmp2 = PyObject_Str(obj);
+\t\tif (tmp2) {
+\t\t\ttmp = PyUnicode_AsASCIIString(tmp2);
+\t\t\tPy_DECREF(tmp2);
+\t\t}
+\t\telse {
+\t\t\ttmp = NULL;
+\t\t}
+\t}
+#else
+\telse {
 \t\ttmp = PyObject_Str(obj);
+\t}
+#endif
 \tif (tmp == NULL) goto capi_fail;
 \tif (*len == -1)
 \t\t*len = PyString_GET_SIZE(tmp);
@@ -667,7 +717,7 @@ static int int_from_pyobj(int* v,PyObject *obj,const char *errmess) {
 \t}
 \tif (PyComplex_Check(obj))
 \t\ttmp = PyObject_GetAttrString(obj,\"real\");
-\telse if (PyString_Check(obj))
+\telse if (PyString_Check(obj) || PyUnicode_Check(obj))
 \t\t/*pass*/;
 \telse if (PySequence_Check(obj))
 \t\ttmp = PySequence_GetItem(obj,0);
@@ -699,7 +749,7 @@ static int long_from_pyobj(long* v,PyObject *obj,const char *errmess) {
 \t}
 \tif (PyComplex_Check(obj))
 \t\ttmp = PyObject_GetAttrString(obj,\"real\");
-\telse if (PyString_Check(obj))
+\telse if (PyString_Check(obj) || PyUnicode_Check(obj))
 \t\t/*pass*/;
 \telse if (PySequence_Check(obj))
 \t\ttmp = PySequence_GetItem(obj,0);
@@ -736,7 +786,7 @@ static int long_long_from_pyobj(long_long* v,PyObject *obj,const char *errmess)
 \t}
 \tif (PyComplex_Check(obj))
 \t\ttmp = PyObject_GetAttrString(obj,\"real\");
-\telse if (PyString_Check(obj))
+\telse if (PyString_Check(obj) || PyUnicode_Check(obj))
 \t\t/*pass*/;
 \telse if (PySequence_Check(obj))
 \t\ttmp = PySequence_GetItem(obj,0);
@@ -763,7 +813,7 @@ static int long_double_from_pyobj(long_double* v,PyObject *obj,const char *errme
 \t\t\treturn 1;
 \t\t}
 \t\telse if (PyArray_Check(obj) && PyArray_TYPE(obj)==PyArray_LONGDOUBLE) {
-\t\t\t(*v) = *((npy_longdouble *)PyArray_DATA(obj))
+\t\t\t(*v) = *((npy_longdouble *)PyArray_DATA(obj));
 \t\t\treturn 1;
 \t\t}
 \t}
@@ -797,7 +847,7 @@ static int double_from_pyobj(double* v,PyObject *obj,const char *errmess) {
 \t}
 \tif (PyComplex_Check(obj))
 \t\ttmp = PyObject_GetAttrString(obj,\"real\");
-\telse if (PyString_Check(obj))
+\telse if (PyString_Check(obj) || PyUnicode_Check(obj))
 \t\t/*pass*/;
 \telse if (PySequence_Check(obj))
 \t\ttmp = PySequence_GetItem(obj,0);
@@ -907,7 +957,7 @@ static int complex_double_from_pyobj(complex_double* v,PyObject *obj,const char
 \t\t(*v).r = PyLong_AsDouble(obj);
 \t\treturn (!PyErr_Occurred());
 \t}
-\tif (PySequence_Check(obj) && (!PyString_Check(obj))) {
+\tif (PySequence_Check(obj) && !(PyString_Check(obj) || PyUnicode_Check(obj))) {
 \t\tPyObject *tmp = PySequence_GetItem(obj,0);
 \t\tif (tmp) {
 \t\t\tif (complex_double_from_pyobj(v,tmp,errmess)) {
@@ -995,7 +1045,7 @@ static int create_cb_arglist(PyObject* fun,PyTupleObject* xa,const int maxnofarg
 \t\t\t\ttot += PyTuple_Size((PyObject *)xa);
 \t\t\ttmp_fun = fun;
 \t\t}
-\t\telse if (PyCObject_Check(fun)) {
+\t\telse if (F2PyCapsule_Check(fun)) {
 \t\t\ttot = maxnofargs;
 \t\t\tif (xa != NULL)
 \t\t\t\text = PyTuple_Size((PyObject *)xa);
@@ -1007,17 +1057,27 @@ static int create_cb_arglist(PyObject* fun,PyTupleObject* xa,const int maxnofarg
 \t\t}
 \t}
 if (tmp_fun==NULL) {
-fprintf(stderr,\"Call-back argument must be function|instance|instance.__call__|f2py-function but got %s.\\n\",(fun==NULL?\"NULL\":fun->ob_type->tp_name));
+fprintf(stderr,\"Call-back argument must be function|instance|instance.__call__|f2py-function but got %s.\\n\",(fun==NULL?\"NULL\":Py_TYPE(fun)->tp_name));
 goto capi_fail;
 }
+#if PY_VERSION_HEX >= 0x03000000
+\tif (PyObject_HasAttrString(tmp_fun,\"__code__\")) {
+\t\tif (PyObject_HasAttrString(tmp = PyObject_GetAttrString(tmp_fun,\"__code__\"),\"co_argcount\"))
+#else
 \tif (PyObject_HasAttrString(tmp_fun,\"func_code\")) {
 \t\tif (PyObject_HasAttrString(tmp = PyObject_GetAttrString(tmp_fun,\"func_code\"),\"co_argcount\"))
+#endif
 \t\t\ttot = PyInt_AsLong(PyObject_GetAttrString(tmp,\"co_argcount\")) - di;
 \t\tPy_XDECREF(tmp);
 \t}
 \t/* Get the number of optional arguments */
+#if PY_VERSION_HEX >= 0x03000000
+\tif (PyObject_HasAttrString(tmp_fun,\"__defaults__\"))
+\t\tif (PyTuple_Check(tmp = PyObject_GetAttrString(tmp_fun,\"__defaults__\")))
+#else
 \tif (PyObject_HasAttrString(tmp_fun,\"func_defaults\"))
 \t\tif (PyTuple_Check(tmp = PyObject_GetAttrString(tmp_fun,\"func_defaults\")))
+#endif
 \t\t\topt = PyTuple_Size(tmp);
 \t\tPy_XDECREF(tmp);
 \t/* Get the number of extra arguments */
@@ -1071,7 +1131,7 @@ def append_needs(need,flag=1):
     if type(need)==types.ListType:
         for n in need:
             append_needs(n,flag)
-    elif type(need)==types.StringType:
+    elif type(need)==str:
         if not need: return
         if need in includes0:
             n = 'includes0'
@@ -1150,7 +1210,8 @@ def get_needs():
                 else:
                     out.append(outneeds[n][0])
                     del outneeds[n][0]
-            if saveout and (0 not in map(lambda x,y:x==y,saveout,outneeds[n])):
+            if saveout and (0 not in map(lambda x,y:x==y,saveout,outneeds[n])) \
+                   and outneeds[n] != []:
                 print n,saveout
                 errmess('get_needs: no progress in sorting needs, probably circular dependence, skipping.\n')
                 out=out+saveout
