@@ -21,7 +21,6 @@ f2py_version = __version__.version
 import sys
 import os
 import pprint
-import shutil
 import types
 import re
 errmess=sys.stderr.write
@@ -31,11 +30,8 @@ show=pprint.pprint
 import crackfortran
 import rules
 import cb_rules
-import common_rules
 import auxfuncs
 import cfuncs
-import capi_maps
-import func2subr
 import f90mod_rules
 
 outmess = auxfuncs.outmess
@@ -69,11 +65,11 @@ Description: This program generates a Python C/API file (<modulename>module.c)
 
 Options:
 
-  --g3-numpy       Use numpy.f2py.lib tool, the 3rd generation of F2PY,
-                   with NumPy support.
   --2d-numpy       Use numpy.f2py tool with NumPy support. [DEFAULT]
   --2d-numeric     Use f2py2e tool with Numeric support.
   --2d-numarray    Use f2py2e tool with Numarray support.
+  --g3-numpy       Use 3rd generation f2py from the separate f2py package.
+                   [NOT AVAILABLE YET]
 
   -h <filename>    Write signatures of the fortran routines to file <filename>
                    and exit. You can then edit <filename> and use it instead
@@ -177,10 +173,9 @@ License:     NumPy license (see LICENSE.txt in the NumPy source code)
 Copyright 1999 - 2005 Pearu Peterson all rights reserved.
 http://cens.ioc.ee/projects/f2py2e/"""%(f2py_version, numpy_version)
 
-
 def scaninputline(inputline):
     files,funcs,skipfuncs,onlyfuncs,debug=[],[],[],[],[]
-    f,f2,f3,f4,f5,f6,f7=1,0,0,0,0,0,0
+    f,f2,f3,f4,f5,f6,f7,f8,f9=1,0,0,0,0,0,0,0,0
     verbose = 1
     dolc=-1
     dolatexdoc = 0
@@ -189,7 +184,9 @@ def scaninputline(inputline):
     buildpath = '.'
     include_paths = []
     signsfile,modulename=None,None
-    options = {'buildpath':buildpath}
+    options = {'buildpath':buildpath,
+               'coutput': None,
+               'f2py_wrapper_output': None}
     for l in inputline:
         if l=='': pass
         elif l=='only:': f=0
@@ -208,6 +205,8 @@ def scaninputline(inputline):
         elif l=='--wrap-functions': wrapfuncs=1
         elif l=='--no-wrap-functions': wrapfuncs=0
         elif l=='--short-latex': options['shortlatex']=1
+        elif l=='--coutput': f8=1
+        elif l=='--f2py-wrapper-output': f9=1
         elif l=='--overwrite-signature': options['h-overwrite']=1
         elif l=='-h': f2=1
         elif l=='-m': f3=1
@@ -228,6 +227,8 @@ def scaninputline(inputline):
         elif f3: f3=0;modulename=l
         elif f6: f6=0;buildpath=l
         elif f7: f7=0;include_paths.extend(l.split(os.pathsep))
+        elif f8: f8=0;options["coutput"]=l
+        elif f9: f9=0;options["f2py_wrapper_output"]=l
         elif f==1:
             try:
                 open(l).close()
@@ -245,7 +246,7 @@ def scaninputline(inputline):
         os.mkdir(buildpath)
     if signsfile:
         signsfile = os.path.join(buildpath,signsfile)
-    if signsfile and os.path.isfile(signsfile) and not options.has_key('h-overwrite'):
+    if signsfile and os.path.isfile(signsfile) and 'h-overwrite' not in options:
         errmess('Signature file "%s" exists!!! Use --overwrite-signature to overwrite.\n'%(signsfile))
         sys.exit()
 
@@ -269,16 +270,16 @@ def callcrackfortran(files,options):
     funcs=[]
     crackfortran.debug=options['debug']
     crackfortran.verbose=options['verbose']
-    if options.has_key('module'):
+    if 'module' in options:
         crackfortran.f77modulename=options['module']
-    if options.has_key('skipfuncs'):
+    if 'skipfuncs' in options:
         crackfortran.skipfuncs=options['skipfuncs']
-    if options.has_key('onlyfuncs'):
+    if 'onlyfuncs' in options:
         crackfortran.onlyfuncs=options['onlyfuncs']
     crackfortran.include_paths[:]=options['include_paths']
     crackfortran.dolowercase=options['do-lower']
     postlist=crackfortran.crackfortran(files)
-    if options.has_key('signsfile'):
+    if 'signsfile' in options:
         outmess('Saving signatures to file "%s"\n'%(options['signsfile']))
         pyf=crackfortran.crack2fortran(postlist)
         if options['signsfile'][-6:]=='stdout':
@@ -287,31 +288,44 @@ def callcrackfortran(files,options):
             f=open(options['signsfile'],'w')
             f.write(pyf)
             f.close()
+    if options["coutput"] is None:
+        for mod in postlist:
+            mod["coutput"] = "%smodule.c" % mod["name"]
+    else:
+        for mod in postlist:
+            mod["coutput"] = options["coutput"]
+    if options["f2py_wrapper_output"] is None:
+        for mod in postlist:
+            mod["f2py_wrapper_output"] = "%s-f2pywrappers.f" % mod["name"]
+    else:
+        for mod in postlist:
+            mod["f2py_wrapper_output"] = options["f2py_wrapper_output"]
     return postlist
 
-def buildmodules(list):
+def buildmodules(lst):
     cfuncs.buildcfuncs()
     outmess('Building modules...\n')
     modules,mnames,isusedby=[],[],{}
-    for i in range(len(list)):
-        if '__user__' in list[i]['name']:
-            cb_rules.buildcallbacks(list[i])
+    for i in range(len(lst)):
+        if '__user__' in lst[i]['name']:
+            cb_rules.buildcallbacks(lst[i])
         else:
-            if list[i].has_key('use'):
-                for u in list[i]['use'].keys():
-                    if not isusedby.has_key(u): isusedby[u]=[]
-                    isusedby[u].append(list[i]['name'])
-            modules.append(list[i])
-            mnames.append(list[i]['name'])
+            if 'use' in lst[i]:
+                for u in lst[i]['use'].keys():
+                    if u not in isusedby:
+                        isusedby[u]=[]
+                    isusedby[u].append(lst[i]['name'])
+            modules.append(lst[i])
+            mnames.append(lst[i]['name'])
     ret = {}
     for i in range(len(mnames)):
-        if isusedby.has_key(mnames[i]):
+        if mnames[i] in isusedby:
             outmess('\tSkipping module "%s" which is used by %s.\n'%(mnames[i],','.join(map(lambda s:'"%s"'%s,isusedby[mnames[i]]))))
         else:
             um=[]
-            if modules[i].has_key('use'):
+            if 'use' in modules[i]:
                 for u in modules[i]['use'].keys():
-                    if isusedby.has_key(u) and u in mnames:
+                    if u in isusedby and u in mnames:
                         um.append(modules[mnames.index(u)])
                     else:
                         outmess('\tModule "%s" uses nonexisting "%s" which will be ignored.\n'%(mnames[i],u))
@@ -321,7 +335,7 @@ def buildmodules(list):
 
 def dict_append(d_out,d_in):
     for (k,v) in d_in.items():
-        if not d_out.has_key(k):
+        if k not in d_out:
             d_out[k] = []
         if type(v) is types.ListType:
             d_out[k] = d_out[k] + v
@@ -332,7 +346,11 @@ def run_main(comline_list):
     """Run f2py as if string.join(comline_list,' ') is used as a command line.
     In case of using -h flag, return None.
     """
-    reload(crackfortran)
+    if sys.version_info[0] >= 3:
+        import imp
+        imp.reload(crackfortran)
+    else:
+        reload(crackfortran)
     f2pydir=os.path.dirname(os.path.abspath(cfuncs.__file__))
     fobjhsrc = os.path.join(f2pydir,'src','fortranobject.h')
     fobjcsrc = os.path.join(f2pydir,'src','fortranobject.c')
@@ -341,23 +359,24 @@ def run_main(comline_list):
     postlist=callcrackfortran(files,options)
     isusedby={}
     for i in range(len(postlist)):
-        if postlist[i].has_key('use'):
+        if 'use' in postlist[i]:
             for u in postlist[i]['use'].keys():
-                if not isusedby.has_key(u): isusedby[u]=[]
+                if u not in isusedby:
+                    isusedby[u]=[]
                 isusedby[u].append(postlist[i]['name'])
     for i in range(len(postlist)):
         if postlist[i]['block']=='python module' and '__user__' in postlist[i]['name']:
-            if isusedby.has_key(postlist[i]['name']):
+            if postlist[i]['name'] in isusedby:
                 #if not quiet:
                 outmess('Skipping Makefile build for module "%s" which is used by %s\n'%(postlist[i]['name'],','.join(map(lambda s:'"%s"'%s,isusedby[postlist[i]['name']]))))
-    if options.has_key('signsfile'):
+    if 'signsfile' in options:
         if options['verbose']>1:
             outmess('Stopping. Edit the signature file and then run f2py on the signature file: ')
             outmess('%s %s\n'%(os.path.basename(sys.argv[0]),options['signsfile']))
         return
     for i in range(len(postlist)):
         if postlist[i]['block']!='python module':
-            if not options.has_key('python module'):
+            if 'python module' not in options:
                 errmess('Tip: If your original code is Fortran source then you must use -m option.\n')
             raise TypeError,'All blocks must be python module blocks but got %s'%(`postlist[i]['block']`)
     auxfuncs.debugoptions=options['debug']
@@ -393,7 +412,7 @@ def run_compile():
     """
     Do it all in one call!
     """
-    import tempfile,os,shutil
+    import tempfile
 
     i = sys.argv.index('-c')
     del sys.argv[i]
@@ -545,6 +564,7 @@ def run_compile():
     setup(ext_modules = [ext])
 
     if remove_build_dir and os.path.exists(build_dir):
+        import shutil
         outmess('Removing build directory %s\n'%(build_dir))
         shutil.rmtree(build_dir)
 
