@@ -4,6 +4,10 @@ import numpy as np
 from numpy.random import randint
 from numpy.lib import Arrayterator
 
+import sys
+if sys.version_info[0] >= 3:
+    from functools import reduce
+
 def test():
     np.random.seed(np.arange(10))
 
@@ -41,3 +45,7 @@ def test():
 
     # Check that all elements are iterated correctly
     assert list(c.flat) == list(d.flat)
+
+if __name__ == '__main__':
+    from numpy.testing import run_module_suite
+    run_module_suite()
