@@ -1,19 +1,22 @@
 """
->>> import numpy.core as nx
->>> from numpy.lib.polynomial import poly1d, polydiv
-
->>> p = poly1d([1.,2,3])
+>>> p = np.poly1d([1.,2,3])
 >>> p
 poly1d([ 1.,  2.,  3.])
->>> print p
+>>> print(p)
    2
 1 x + 2 x + 3
->>> q = poly1d([3.,2,1])
+>>> q = np.poly1d([3.,2,1])
 >>> q
 poly1d([ 3.,  2.,  1.])
->>> print q
+>>> print(q)
    2
 3 x + 2 x + 1
+>>> print(np.poly1d([1.89999+2j, -3j, -5.12345678, 2+1j]))
+            3      2
+(1.9 + 2j) x - 3j x - 5.123 x + (2 + 1j)
+>>> print(np.poly1d([-3, -2, -1]))
+    2
+-3 x - 2 x - 1
 
 >>> p(0)
 3.0
@@ -40,7 +43,7 @@ poly1d([  9.,  12.,  16.,   8.,   6.])
 >>> q(p)
 poly1d([  3.,  12.,  32.,  40.,  34.])
 
->>> nx.asarray(p)
+>>> np.asarray(p)
 array([ 1.,  2.,  3.])
 >>> len(p)
 2
@@ -60,16 +63,16 @@ poly1d([ 2.,  2.])
 >>> p.deriv(2)
 poly1d([ 2.])
 
->>> q = poly1d([1.,2,3], variable='y')
->>> print q
+>>> q = np.poly1d([1.,2,3], variable='y')
+>>> print(q)
    2
 1 y + 2 y + 3
->>> q = poly1d([1.,2,3], variable='lambda')
->>> print q
+>>> q = np.poly1d([1.,2,3], variable='lambda')
+>>> print(q)
         2
 1 lambda + 2 lambda + 3
 
->>> polydiv(poly1d([1,0,-1]), poly1d([1,1]))
+>>> np.polydiv(np.poly1d([1,0,-1]), np.poly1d([1,1]))
 (poly1d([ 1., -1.]), poly1d([ 0.]))
 """
 
@@ -110,6 +113,35 @@ class TestDocs(TestCase):
         cc = np.concatenate((c,c), axis=1)
         assert_almost_equal(cc, np.polyfit(x,yy,2))
 
+    def test_objects(self):
+        from decimal import Decimal
+        p = np.poly1d([Decimal('4.0'), Decimal('3.0'), Decimal('2.0')])
+        p2 = p * Decimal('1.333333333333333')
+        assert p2[1] == Decimal("3.9999999999999990")
+        p2 = p.deriv()
+        assert p2[1] == Decimal('8.0')
+        p2 = p.integ()
+        assert p2[3] == Decimal("1.333333333333333333333333333")
+        assert p2[2] == Decimal('1.5')
+        assert np.issubdtype(p2.coeffs.dtype, np.object_)
+
+    def test_complex(self):
+        p = np.poly1d([3j, 2j, 1j])
+        p2 = p.integ()
+        assert (p2.coeffs == [1j,1j,1j,0]).all()
+        p2 = p.deriv()
+        assert (p2.coeffs == [6j,2j]).all()
+
+    def test_integ_coeffs(self):
+        p = np.poly1d([3,2,1])
+        p2 = p.integ(3, k=[9,7,6])
+        assert (p2.coeffs == [1/4./5.,1/3./4.,1/2./3.,9/1./2.,7,6]).all()
+
+    def test_zero_dims(self):
+        try:
+            np.poly(np.zeros((0, 0)))
+        except ValueError:
+            pass
 
 if __name__ == "__main__":
     run_module_suite()
